package org.artifactory.ph;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.common.repo.RepoInfo;
import org.artifactory.config.ConfigurationChangesInterceptor;
import com.jfrog.ph.acl.listener.PackageConfigChangedEventListener;
import com.jfrog.ph.acl.listener.PackageConfigChangedEventType;
import com.jfrog.ph.acl.listener.PackageRepoConfigListenerContext;
import com.jfrog.ph.acl.service.PackageRepoService;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.repo.RepoConfigChange;
import org.artifactory.repo.RepoDetailsType;
import org.artifactory.repo.interceptor.RepoConfigChangeInterceptor;
import java.util.List;
import java.util.Map;

@Slf4j
public class PackageConfigurationChangesInterceptorImpl implements ConfigurationChangesInterceptor,
        RepoConfigChangeInterceptor {

    private final PackageConfigChangedEventListener packageConfigChangedEventListener;

    private final PackageRepoService packageRepoService;

    public PackageConfigurationChangesInterceptorImpl(
            PackageConfigChangedEventListener packageConfigChangedEventListener,
            PackageRepoService packageRepoService) {
        this.packageConfigChangedEventListener = packageConfigChangedEventListener;
        this.packageRepoService = packageRepoService;
    }

    @Override
    public void onAfterSave(CentralConfigDescriptor newDescriptor, CentralConfigDescriptor oldDescriptor) {
        for (PackageConfigChangedEventType packageConfigChangedEventType : PackageConfigChangedEventType.values()) {
            try {
                PackageConfigChangeType packageConfigChangeType =
                        PackageConfigChangeType.valueOf(packageConfigChangedEventType.name());
                if (packageConfigChangeType.isChanged(newDescriptor, oldDescriptor)) {
                    packageConfigChangedEventListener.onGlobalConfigChange(packageConfigChangedEventType);
                }
            } catch (IllegalArgumentException e) {
                log.debug("Skipping on {} - not found", packageConfigChangedEventType.name());
            }
        }
    }

    @Override
    public void onAfterSave(@NonNull Map<RepoDetailsType, List<RepoConfigChange>> repoConfigChanges) {
        for (Map.Entry<RepoDetailsType, List<RepoConfigChange>> packageConfigChangedEventType :
                repoConfigChanges.entrySet()) {
            List<RepoConfigChange> repoChangeList = packageConfigChangedEventType.getValue();
            for (RepoConfigChange repo : repoChangeList) {
                try {
                    if (repo.getConfigAction() != null) {
                        PackageConfigChangedEventType eventType = PackageConfigChangedEventType.getEventType(
                                repo.getConfigAction().name());
                        if (eventType != null) {
                            RepoInfo repoInfo = packageRepoService.getRepoInfo(repo.getRepoKey());
                            PackageRepoConfigListenerContext context = new PackageRepoConfigListenerContext(
                                    repo.getRepoKey(), repoInfo, eventType);
                            packageConfigChangedEventListener.onRepoConfigChange(context);
                        }
                    }
                } catch (Exception e) {
                    log.error("Skipping on {} - not found, {}", repo.getRepoKey(), e.getMessage());
                    log.debug("Skipping on {} - not found", repo.getRepoKey(), e);
                }
            }
        }
    }
}
