package org.artifactory.repo.converter;

import com.google.common.collect.Lists;
import org.artifactory.repo.converter.v1.TerraformEnableBypassHeaderConverter;

import java.util.Arrays;
import java.util.List;

/**
 * @author Tamar Rosen
 */
public enum RepositoryConfigConverterEnum {
    V1(1, new TerraformEnableBypassHeaderConverter());
    private long version;
    private final RepositoriesConverter[] repositoriesConverter;

    RepositoryConfigConverterEnum(long version, RepositoriesConverter... repositoriesConverters) {
        this.version = version;
        this.repositoriesConverter = repositoriesConverters;
    }

    public static List<RepositoriesConverter> getRelevantConverters(long fromVersion) {
        List<RepositoriesConverter> converters = Lists.newArrayList();
        for (RepositoryConfigConverterEnum converterVersion : RepositoryConfigConverterEnum.values()) {
            if (converterVersion.version > fromVersion) {
                converters.addAll(Arrays.asList(converterVersion.getRepositoriesConverter()));
            }
        }
        return converters;
    }

    public RepositoriesConverter[] getRepositoriesConverter() {
        return repositoriesConverter;
    }
}
