package org.artifactory.queue.behaviors;

import com.google.common.collect.Lists;
import org.artifactory.event.queue.QueueReductionBehaviours;
import org.artifactory.queue.QueueEventImpl;
import org.testng.annotations.Test;

import java.util.List;

import static org.artifactory.event.EventType.create;
import static org.artifactory.event.EventType.delete;
import static org.artifactory.event.EventType.props;
import static org.artifactory.event.EventType.update;
import static org.testng.Assert.assertEquals;

@Test
public class QueueReductionBehaviourTest extends AbstractEventReductionBehaviorTest {

    public void chainBasicBehaviours() {
        QueueReductionBehaviours behaviors = QueueReductionBehaviours.Basic;

        List<QueueEventImpl> input;
        List<QueueEventImpl> output;

        input = Lists.newArrayList();
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        output = QueueReductionBehavioursImpl.get(behaviors).applyChain(input);
        assertEquals(output.size(), 0);

        input = Lists.newArrayList();
        add(input, delete, "a"); //because of CreateEliminateDeleteBehaviour
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        output = QueueReductionBehavioursImpl.get(behaviors).applyChain(input);
        assertEquals(output.size(), 0);

        reset();
        input = Lists.newArrayList();
        add(input, create, "a"); //removed 2nd because of BasicEventDedupBehaviour
        add(input, delete, "a"); //removed 1st because of CreateEliminateDeleteBehaviour
        add(input, create, "a");
        output = QueueReductionBehavioursImpl.get(behaviors).applyChain(input);
        assertEquals(output.size(), 1);
        assertEquals(output.get(0).toString(), "3|create|a");
    }


    public void chainIncrementalIndexBehaviours() {
        QueueReductionBehaviours behaviors = QueueReductionBehaviours.IncrementalIndex;

        List<QueueEventImpl> input;
        List<QueueEventImpl> output;

        input = Lists.newArrayList();
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        output = QueueReductionBehavioursImpl.get(behaviors).applyChain(input);
        assertEquals(output.size(), 0);

        input = Lists.newArrayList();
        add(input, delete, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, create, "a"); //because of DeleteEliminateUpdateOrCreateBehaviour
        add(input, delete, "a");
        output = QueueReductionBehavioursImpl.get(behaviors).applyChain(input);
        assertEquals(output.size(), 1);
        assertEquals(output.get(0).toString(), "3|delete|a");

        reset();
        input = Lists.newArrayList();
        add(input, create, "a");
        add(input, delete, "a"); //removed 1st because of DeleteEliminateUpdateOrCreateBehavior
        add(input, create, "a"); //removed 2nd because of FederatedDedupBehavior
        output = QueueReductionBehavioursImpl.get(behaviors).applyChain(input);
        assertEquals(output.size(), 1);
        assertEquals(output.get(0).toString(), "3|create|a");

        reset();
        input = Lists.newArrayList();
        add(input, update, "b");
        add(input, create, "a");
        add(input, props, "a");
        add(input, props, "a");
        add(input, props, "b");
        add(input, update, "a");
        add(input, update, "a");
        add(input, props, "a");
        output = QueueReductionBehavioursImpl.get(behaviors).applyChain(input);
        assertEquals(output.size(), 2);
        assertEquals(output.get(0).toString(), "5|update|b", "regular prioritize dedup");
        assertEquals(output.get(1).toString(), "8|create|a", "merged prioritize dedup");

        assertEquals(output.get(1).getOriginalEvent().getEventId(), 2);
    }

}
