package org.artifactory.converter.mirror;

import org.artifactory.common.config.NegotiationException;
import org.artifactory.model.FederatedRepoConfig;
import org.artifactory.version.CompoundVersionDetails;

/**
 * @author Alexei Vainshtein
 */
public interface MetadataNegotiationService {

    /**
     * Performs negotiation for federated repositories if needed upon upgrade
     * @param source - The version of Artifactory
     */
    default void negotiateUponUpgrade(CompoundVersionDetails source) {}

    /**
     * Performs negotiation for federated repositories if needed per given repository
     * @param newRepoConfig - The new federated descriptor to be negotiated.
     * @param oldRepoConfig - The old descriptor. This required to fetch information during negotiation conflict
     * @return the descriptor with changes if were made after the negotiation
     * @throws NegotiationException upon error during the negotiation
     */
    default FederatedRepoConfig negotiatePerRepo(FederatedRepoConfig newRepoConfig,
            FederatedRepoConfig oldRepoConfig) throws NegotiationException {
        return null;
    }
}
