package org.artifactory.storage.fs.service;

import org.jfrog.common.ArgUtils;

/**
 * Keys enum and conventions for using ConfigsService
 *
 * @author Avishay Halpren
 */
public enum ConfigsServiceKey {

    /**
     * naming conventions for keys:
     *  "domain-namespace.meaningful.key.name"
     *      domain-namespace can be: artifactory, storage, packages ,bundle ...
     *      e.g: artifactory.gc.iteration, packages.some.value
     *  for a key which represent a config object (translated to json) use .config suffix
     *      e.g: storage.checksum.replication.config will be translated to a specific Object (not primitive)
     *  for a primitive value no .suffix is required
     *      e.g: artifactory.gc.iteration will just an int representation
     */

    STORAGE_CHECKSUM_REPLICATION("storage.checksum.replication.config"),
    ARTIFACTORY_GC_ITERATION("artifactory.gc.iteration"),
    ;

    //
    private final String keyName;

    ConfigsServiceKey(String keyName) {
        this.keyName = ArgUtils.requireMaxLength(keyName, 256, "key max length must be lower than 256 characters");
    }

    public String getKeyName() {
        return keyName;
    }
}
