package org.artifactory.logging.layout.request;

import io.vavr.collection.Array;
import org.artifactory.logging.LoggingTestUtils;
import org.artifactory.logging.SaasLogNames;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.MDC;

import java.time.Instant;

/**
 * @author Tomer Slutsky
 */
public class OutboundRequestJsonLayoutTest {
    OutboundRequestJsonLayout outboundRequestJsonLayout = new OutboundRequestJsonLayout();

    @Test
    public void testDoLayout() {
        MDC.put("uber-trace-id", "traceid");
        String s = outboundRequestJsonLayout.doLayout(LoggingTestUtils
                .createEvent(Array.of(Instant.ofEpochMilli(System.currentTimeMillis()).toString(),
                                "traceid", "remote-repo", "", "HEAD", "http://localhost:8081", "200", 1996, 2010, 25),
                        OutboundRequestJsonLayoutTest.class));
        Assertions.assertTrue(s.contains("\"" + SaasLogNames.OUTBOUND_REQUEST_LOG.getLogKey() + "\":\"" +
                SaasLogNames.OUTBOUND_REQUEST_LOG.getLogName() + "\""));
        Assertions.assertTrue(s.contains("\"trace-id\":\"traceid\""));
        Assertions.assertTrue(s.contains("\"remote_repository\":\"remote-repo\""));
        Assertions.assertTrue(s.contains("\"path\":\"http://localhost:8081\""));
        Assertions.assertTrue(s.contains("\"user\":\"\""));
        Assertions.assertTrue(s.contains("\"method\":\"HEAD\""));
        Assertions.assertTrue(s.contains("\"request\":{"));
        Assertions.assertTrue(s.contains("\"response\":{"));
        Assertions.assertTrue(s.contains("\"datetime\":"));
        Assertions.assertTrue(s.contains("\"length\":2010"));
        Assertions.assertTrue(s.contains("\"length\":1996"));
        Assertions.assertTrue(s.contains("\"duration\":25"));
    }
}