package org.artifactory.config;

import lombok.experimental.UtilityClass;

/**
 * Holds the keys of the access config as it comes from the patch yaml
 *
 * @see
 * <a href="https://www.jfrog.com/confluence/display/JFROG/Artifactory+YAML+Configuration">Artifactory YAML Configuration</a>
 * @see org.artifactory.rest.resource.system.ConfigResource#patchConfig
 * @author Mor Merhav
 */
@UtilityClass
public class AccessConfigKeys {

    public static final String ACCESS_ADOPTION_CONFIG = "adoption.config";
    public static final String ACCESS_ADOPTION_CONFIG_DISABLED = "disabled";

    public static final String SECURITY = "security";

    // Directly under security
    public static final String SECURITY_ANON_ACCESS_ENABLED = "anonAccessEnabled";
    public static final String SECURITY_USER_LOCK_POLICY = "userLockPolicy";
    public static final String SECURITY_PASSWORD_SETTINGS = "passwordSettings";
    public static final String SECURITY_BASIC_AUTH_ENABLED = "basicAuthEnabled";

    // Directly under password settings
    public static final String PASSWORD_SETTINGS_ENCRYPTION_POLICY = "encryptionPolicy";


    // Directly under user lock policy
    public static final String USER_LOCK_POLICY_LOGIN_ATTEMPTS = "loginAttempts";
    public static final String USER_LOCK_POLICY_ENABLED = "enabled";

    public static final String HTTP_SSO_SETTINGS = "httpSsoSettings";
    public static final String HTTP_SSO_PROXIED = "httpSsoProxied";
    public static final String NO_AUTO_CREATION = "noAutoUserCreation";
    public static final String ALLOW_USER_TO_ACCESS_PROFILE = "allowUserToAccessProfile";
    public static final String REMOTE_USER_REQUEST_VARIABLE = "remoteUserRequestVariable";
    public static final String REMOTE_USER = "REMOTE_USER";
    public static final String SYNC_LDAP_GROUPS = "syncLdapGroups";
}