package org.artifactory.model;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.artifactory.repo.RepoDetailsType;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static org.artifactory.repo.RepoDetailsType.FEDERATED;
import static org.artifactory.repo.RepoDetailsType.LOCAL;
import static org.artifactory.repo.RepoDetailsType.RELEASE_BUNDLE;
import static org.artifactory.repo.RepoDetailsType.REMOTE;
import static org.artifactory.repo.RepoDetailsType.VIRTUAL;

/**
 * @author Tamar Rosen
 */
@Data
@NoArgsConstructor
public class AggregatedReposConfig {
    List<LocalRepoConfig> localRepoConfigs;
    List<FederatedRepoConfig> federatedRepoConfigs;
    List<ReleaseBundleRepoConfig> releaseBundleRepoConfigs;
    List<RemoteRepoConfig> remoteRepoConfigs;
    List<VirtualRepoConfig> virtualRepoConfigs;

    public AggregatedReposConfig(Collection<RepoConfig> allRepositoryConfigs) {
        setLocalRepoConfigs(
                filterRepoConfigsByType(allRepositoryConfigs, LOCAL, LocalRepoConfig.class));
        setFederatedRepoConfigs(
                filterRepoConfigsByType(allRepositoryConfigs, FEDERATED, FederatedRepoConfig.class));
        setReleaseBundleRepoConfigs(
                filterRepoConfigsByType(allRepositoryConfigs, RELEASE_BUNDLE, ReleaseBundleRepoConfig.class));
        setRemoteRepoConfigs(
                filterRepoConfigsByType(allRepositoryConfigs, REMOTE, RemoteRepoConfig.class));
        setVirtualRepoConfigs(
                filterRepoConfigsByType(allRepositoryConfigs, VIRTUAL, VirtualRepoConfig.class));
    }

    public List<RepoConfig> getAll() {
        List<RepoConfig> all = new ArrayList<>();
        if (localRepoConfigs != null) {
            all.addAll(localRepoConfigs);
        }
        if (federatedRepoConfigs != null) {
            all.addAll(federatedRepoConfigs);
        }
        if (releaseBundleRepoConfigs != null) {
            all.addAll(releaseBundleRepoConfigs);
        }
        if (remoteRepoConfigs != null) {
            all.addAll(remoteRepoConfigs);
        }
        if (virtualRepoConfigs != null) {
            all.addAll(virtualRepoConfigs);
        }
        return all;
    }

    @NotNull
    private <T> List<T> filterRepoConfigsByType(Collection<RepoConfig> allRepositoryConfigs,
            RepoDetailsType repoType, Class<T> repoConfigTypedClass) {
        return allRepositoryConfigs.stream()
                .filter(repoConfig -> repoType.equals(repoConfig.getRepoType()))
                .map(repoConfigTypedClass::cast)
                .toList();
    }
}
