package org.artifactory.repo.federated;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonSetter;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Getter;
import org.artifactory.common.config.metadata.MetadataInfo;

import java.util.Objects;

/**
 * @author Tamar Rosen
 */
@Data
public class FederatedMember {
    public static final String BASE_URL_PLACEHOLDER = "<base-url-placeholder>";
    @MetadataInfo
    @Getter(value = AccessLevel.NONE)
    private String url;
    @MetadataInfo
    private String mode;
    @MetadataInfo(defaultValue = "true")
    private boolean enabled = true;

    public FederatedMember() {
    }

    public FederatedMember(FederatedMember federatedMember) {
        this.url = federatedMember.getUrlOrTemplate();
        this.mode = federatedMember.getMode();
        this.enabled = federatedMember.isEnabled();
    }

    @Override
    public int hashCode() {
        return Objects.hash(url, enabled);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        FederatedMember that = (FederatedMember) o;
        if (enabled != that.enabled) {
            return false;
        }
        return Objects.equals(url, that.url);
    }

    @Override
    public String toString() {
        return "FederatedMember{" +
                "url='" + url + '\'' +
                ", enabled=" + enabled +
                '}';
    }

    /**
     * Returns the federated member URL that may or may not contain a baseURL placeholder (template), depending on
     * the type of federated member (remote, local)
     * The placeholder URL that is returned from this method will be resolved dynamically, rather than containing a
     * strict base URL string.
     *
     * @return the full federated member URL for remote members, and a URL template for the local member.
     */
    @JsonProperty("url")
    public String getUrlOrTemplate() {
        return url;
    }

    @JsonSetter("url")
    public void setUrlOrTemplate(String url) {
        this.url = url;
    }
}
