package org.artifactory.util;

import com.google.common.collect.Maps;
import org.artifactory.repo.packagetype.configuration.GoConfiguration;
import org.artifactory.repo.vcs.VcsGitProvider;
import org.artifactory.repo.vcs.VcsType;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.util.Map;

/**
 * @author Tamar Rosen
 */
class GoPackageTypeConfigMapperTest {
    @Test
    void testStandardMapping() {
        Map<String, String> packageTypeConfig =
                Map.of(GoPackageTypeConfigMapper.MODEL_VERSION, "1",
                        ExternalDependenciesPackageTypeConfigMapper.EXTERNAL_DEPENDENCIES_ENABLED, "true",
                        ExternalDependenciesPackageTypeConfigMapper.EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF,
                        "remote-repo",
                        ExternalDependenciesPackageTypeConfigMapper.EXTERNAL_DEPENDENCIES_PATTERNS,
                        "[\"1\",\"2\",\"3\"]",
                        VcsPackageTypeConfigMapper.VCS_GIT_PROVIDER, "BITBUCKET",
                        VcsPackageTypeConfigMapper.VCS_GIT_DOWNLOAD_URL, "git-download-url.com",
                        VcsPackageTypeConfigMapper.VCS_TYPE, "GIT"
                );

        GoConfiguration GoConfiguration = GoPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
        Assertions.assertTrue(GoConfiguration.getExternalDependencies().isEnabled());
        Assertions.assertEquals(1, GoConfiguration.getModelVersion());
        Assertions.assertEquals("remote-repo", GoConfiguration.getExternalDependencies().getRemoteRepoRef());
        Assertions.assertEquals("1", GoConfiguration.getExternalDependencies().getPatterns().get(0));
        Assertions.assertEquals("2", GoConfiguration.getExternalDependencies().getPatterns().get(1));
        Assertions.assertEquals("3", GoConfiguration.getExternalDependencies().getPatterns().get(2));
        Assertions.assertEquals(VcsGitProvider.BITBUCKET, GoConfiguration.getVcs().getGit().getProvider());
        Assertions.assertEquals("git-download-url.com", GoConfiguration.getVcs().getGit().getDownloadUrl());
        Assertions.assertEquals(VcsType.GIT, GoConfiguration.getVcs().getType());
    }

    @Test
    void testAllNullsMapping() {
        Map<String, String> packageTypeConfig = Maps.newHashMap();
        packageTypeConfig.put(GoPackageTypeConfigMapper.MODEL_VERSION, null);
        packageTypeConfig.put(ExternalDependenciesPackageTypeConfigMapper.EXTERNAL_DEPENDENCIES_ENABLED, null);
        packageTypeConfig.put(ExternalDependenciesPackageTypeConfigMapper.EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF, null);
        packageTypeConfig.put(ExternalDependenciesPackageTypeConfigMapper.EXTERNAL_DEPENDENCIES_PATTERNS, null);
        packageTypeConfig.put(VcsPackageTypeConfigMapper.VCS_GIT_PROVIDER, null);
        packageTypeConfig.put(VcsPackageTypeConfigMapper.VCS_GIT_DOWNLOAD_URL, null);
        packageTypeConfig.put(VcsPackageTypeConfigMapper.VCS_TYPE, null);

        GoConfiguration GoConfiguration = GoPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);

        Assertions.assertFalse(GoConfiguration.getExternalDependencies().isEnabled());
        Assertions.assertEquals(0L, GoConfiguration.getModelVersion());
        Assertions.assertNull(GoConfiguration.getExternalDependencies().getRemoteRepoRef());
        Assertions.assertNull(GoConfiguration.getExternalDependencies().getPatterns());
        Assertions.assertEquals(VcsGitProvider.GITHUB, GoConfiguration.getVcs().getGit().getProvider());
        Assertions.assertNull(GoConfiguration.getVcs().getGit().getDownloadUrl());
        Assertions.assertEquals(VcsType.GIT, GoConfiguration.getVcs().getType());
    }

    @Test
    void testEmptyListMapping() {
        Map<String, String> packageTypeConfig = Maps.newHashMap();
        packageTypeConfig.put(ExternalDependenciesPackageTypeConfigMapper.EXTERNAL_DEPENDENCIES_PATTERNS, "[]");

        GoConfiguration bowerConfiguration = GoPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);

        Assertions.assertNotNull(bowerConfiguration.getExternalDependencies().getPatterns());
        Assertions.assertTrue(bowerConfiguration.getExternalDependencies().getPatterns().isEmpty());
    }
}