/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2019 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.binstore.service;

import com.google.common.collect.ImmutableList;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.storage.GCCandidate;
import org.artifactory.storage.binstore.service.cleanup.GCFunction;
import org.artifactory.storage.binstore.service.cleanup.GCProvider;
import org.artifactory.storage.binstore.service.cleanup.GCProviderWrapper;
import org.artifactory.storage.binstore.service.cleanup.GarbageCollectorStrategy;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.storage.binstore.ifc.GarbageCollectorInfo;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author dudim
 */
@Test
public class GCProviderWrapperTest extends ArtifactoryHomeBoundTest {

    public void testErrorCount() {
        ArtifactorySystemProperties artifactoryProperties = ArtifactoryHome.get().getArtifactoryProperties();
        artifactoryProperties.setProperty(ConstantValues.gcFailCountThreshold.getPropertyName(), "3");
        GCProvider gcProviderMock = mock(GCProvider.class);

        when(gcProviderMock.getBatch()).thenReturn(ImmutableList.of(
                new GCCandidate(null, "isEnabled=true", null, null, 0)));

        when(gcProviderMock.getAction()).thenReturn((o, o2) -> {
            throw new RuntimeException("increasing error count");
        });

        GCProviderWrapper gcProviderWrapper = new GCProviderWrapper(gcProviderMock);
        assertEquals(gcProviderWrapper.getBatch().get(0).sha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0),
                        new GarbageCollectorInfo(GarbageCollectorStrategy.TRASH_AND_BINARIES.name(), 10));
        assertEquals(gcProviderWrapper.getBatch().get(0).sha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0),
                        new GarbageCollectorInfo(GarbageCollectorStrategy.TRASH_AND_BINARIES.name(), 10));
        assertEquals(gcProviderWrapper.getBatch().get(0).sha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0),
                        new GarbageCollectorInfo(GarbageCollectorStrategy.TRASH_AND_BINARIES.name(), 10));
        assertTrue(gcProviderWrapper.getBatch().isEmpty());
    }

    @Test
    public void getActionTest() {
        GCProvider gcProviderMock = mock(GCProvider.class);
        GCProviderWrapper gcProviderWrapper = new GCProviderWrapper(gcProviderMock);
        GCFunction action = gcProviderWrapper.getAction();
        GarbageCollectorInfo info =
                new GarbageCollectorInfo(GarbageCollectorStrategy.TRASH_AND_BINARIES.name(), 10);
        GCCandidate gcCandidate = new GCCandidate(null, null, "t", null, 0);
        action.accept(gcCandidate, info);
        assertEquals(info.getFailures(), 1);

        when(gcProviderMock.getAction()).thenReturn((candidate, result) -> true);
        assertTrue(action.accept(gcCandidate, info));
        assertEquals(info.getFailures(), 1);
    }
}