package org.artifactory.storage.db.repo.service.mapper;

import org.artifactory.model.RepoConfig;
import org.artifactory.storage.db.repo.entity.DbRepositoryConfig;
import org.jfrog.common.JsonUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

/**
 * @author Tamar Rosen
 */
@Mapper(uses = RepositoryConfigAttributesMapper.class)
public interface RepositoryConfigMapper {

    @Mapping(source = "repoType",target = "type",
            qualifiedByName = {"RepositoryConfigAttributesMapper", "repoDetailTypeToLowercaseRepoTypeName"})
    @Mapping(source = "packageType",target = "packageType",
            qualifiedByName = {"RepositoryConfigAttributesMapper", "packageTypeToTypeString"})
    @Mapping(source = "baseConfig",target = ".")
    @Mapping(source = "modification", target = "modified")
    @Mapping(source = ".", target = "repositoryInfoBlob")
    DbRepositoryConfig repositoryConfigToDbRepositoryConfig(RepoConfig repositoryConfig);

    default byte[] repositoryConfigToDbInfoBlob(RepoConfig repositoryConfig) {
        if (repositoryConfig != null) {
            return JsonUtils.getInstance().valueToByteArray(repositoryConfig);
        }
        return new byte[0];
    }

    default RepoConfig dbInfoBlobToRepositoryConfig(DbRepositoryConfig dbRepositoryConfig) {
        if (dbRepositoryConfig != null) {
            RepoConfig repoConfig =
                    JsonUtils.getInstance().readValue(dbRepositoryConfig.getRepositoryInfoBlob(), RepoConfig.class);
            repoConfig.setModification(dbRepositoryConfig.getModified());
            return repoConfig;
        }
        return null;
    }
}