package org.artifactory.storage.db;

import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.concurrent.ArtifactoryExecutor;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.artifactory.security.access.AccessService;
import org.artifactory.storage.db.binstore.BinaryStoreConfigServiceImpl;
import org.artifactory.storage.spring.ArtifactoryStorageContext;
import org.jfrog.storage.JdbcHelper;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.context.ApplicationEventPublisher;

import java.util.concurrent.CountDownLatch;

import static org.artifactory.concurrent.ArtifactoryExecutorFactory.newFixedThreadPool;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author TamirH
 */
class BinaryStoreConfigServiceImplTest {

    @Test
    void createContextBinderFunc() throws InterruptedException {
        ArtifactoryStorageContext context = mock(ArtifactoryStorageContext.class);
        ArtifactoryTracer artifactoryTracer = mock(ArtifactoryTracer.class);
        BinaryStoreConfigServiceImpl binaryStoreConfigService =
                new BinaryStoreConfigServiceImpl(mock(JdbcHelper.class), artifactoryTracer, mock(
                        ApplicationEventPublisher.class));
        binaryStoreConfigService.setAccessService(mock(AccessService.class));
        binaryStoreConfigService.setInternalDbService(mock(InternalDbService.class));
        when(context.beanForType(ArtifactoryTracer.class)).thenReturn(artifactoryTracer);
        ArtifactoryContextThreadBinder.bind(context);
        ArtifactoryExecutor artifactoryExecutor = newFixedThreadPool(1, context, false);
        try {
            CountDownLatch latch = new CountDownLatch(1);
            artifactoryExecutor.submit(() -> {
                //contextBinderFunc.bind();
                Assertions.assertEquals(context, ContextHelper.get());
                latch.countDown();
            });
            latch.await();
        } finally {
            ArtifactoryContextThreadBinder.unbind();
            artifactoryExecutor.shutdownNow();

        }
    }
}