package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.jfrog.converter.DbVersion;
import org.jfrog.converter.model.ChangeLogContext;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import static org.artifactory.storage.db.util.ArtifactoryDbVersionUtil.versionsBetween;

/**
 * @author Inbar Tal
 */
@Test
public class V38UpgradeSetStoringRepoTest extends UpgradeBaseTest {

    @BeforeMethod
    public void setup() throws IOException, SQLException {
        resetToVersion(ArtifactoryDBVersion.v33);
        importSql("/sql/bundles_non_default_storing_repo.sql");
        final List<DbVersion> dbVersions = versionsBetween(ArtifactoryDBVersion.v33, ArtifactoryDBVersion.v40);
        for (DbVersion dbVersion : dbVersions) {
            try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                    dbProperties.getDbType(), dbVersion)) {
                dbVersion.converter().convert(context);
            }
        }
    }

    public void testStoringRepoSetToDefaultOnlyIfTypeIsTarget() throws SQLException {
        try (ResultSet res = jdbcHelper.executeSelect("SELECT * FROM artifact_bundles")) {
            while (res.next()) {
                String type = res.getString("type");
                String storingRepo = res.getString("storing_repo");
                if ("SOURCE".equals(type)) {
                    Assert.assertEquals(storingRepo, "my-repo");
                } else {
                    Assert.assertEquals(type, "TARGET");
                    Assert.assertEquals(storingRepo, "release-bundles");
                }
            }
        }
    }

    @Override
    protected ArtifactoryDBVersion upToVersion() {
        return ArtifactoryDBVersion.v38;
    }
}

