package org.artifactory.rest.common.exception.mapper;

import com.jfrog.bh.acl.error.base.BundleException;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.artifactory.rest.ErrorResponse;
import org.springframework.stereotype.Component;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;

import static com.jfrog.bh.util.BundleLoggingUtils.logBundleException;

/**
 * Acts as a controller advice for all Bundle-Handler REST resources.
 * <p>
 * All instances of {@link BundleException}, which are thrown up to
 * the REST controllers are intercepted at this point.
 * <p>
 * HTTP clients get only a status code and a public message from the exception.
 * <p>
 * The log message varies based on the exception subtype and the log level enabled.
 */
@Slf4j
@Provider
@Component
public class BundleExceptionMapper implements ExceptionMapper<BundleException> {

    @NonNull
    @Override
    public Response toResponse(@NonNull BundleException e) {
        logBundleException(e, log);
        return buildErrorResponse(e);
    }

    @NonNull
    private Response buildErrorResponse(@NonNull BundleException e) {
        return Response.status(e.getStatus())
                .type(MediaType.APPLICATION_JSON)
                .entity(new ErrorResponse(e.getStatus(), e.getPublicMessage()))
                .build();
    }
}
