package org.artifactory.rest.common.validator;

import org.artifactory.api.config.RepositoryConfigService;
import org.artifactory.rest.common.exception.RepoConfigException;
import org.testng.Assert;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/*
 * @author Omri Ziv
 */
public class RepositoryNameRestValidatorTest {

    private final RepositoryConfigService repositoryConfigService = mock(RepositoryConfigService.class);

    @Test
    public void testValidRepoName() throws RepoConfigException {
        when(repositoryConfigService.existsRepositoryIgnoreCase(anyString())).thenReturn(false);
        RepositoryNameRestValidator.validateRepoName("valid-repo-name", "remote", repositoryConfigService);
    }

    @Test(dataProvider = "invalidRepoNames")
    public void testInvalidRepoName(String repoName) {
        when(repositoryConfigService.existsRepositoryIgnoreCase(anyString())).thenReturn(false);
        Assert.assertThrows(RepoConfigException.class,
                () -> RepositoryNameRestValidator.validateRepoName(repoName, "local", repositoryConfigService));
    }

    @Test(expectedExceptions = RepoConfigException.class)
    public void testKeyUnavailableInRepositoryConfigService() throws RepoConfigException {
        when(repositoryConfigService.existsRepositoryIgnoreCase(anyString())).thenReturn(true);
        RepositoryNameRestValidator.validateRepoName("valid-repo-name", "remote", repositoryConfigService);
    }

    @DataProvider(name = "invalidRepoNames")
    public static Object[][] repoNames() {
        return new Object[][] {
                {"jfrog-artifactory-system-repo-name"},
                {"jfrog-system-reserved-repo-name"},
                {"some-repo-?-name"},
                {"/some-repo-name"},
                {"some-repo-name*"},
                {""},
                {null},
                {"very-very-l0000000oooooooooooooooooooooooooooooooooooong-local-repo"},
                {"_intransit"},
                {"artifactory-pipe-info"},
                {"   leading-space-repo"},
                {"trailing-space-repo   "}
        };
    }
}
