/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.p2.P2Repo;
import org.artifactory.addon.webstart.ArtifactWebstartAddon;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.config.RepositoryConfigService;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.property.PropertySet;
import org.artifactory.descriptor.replication.LocalReplicationDescriptor;
import org.artifactory.descriptor.replication.RemoteReplicationDescriptor;
import org.artifactory.descriptor.repo.ProxyDescriptor;
import org.artifactory.descriptor.repo.RepoLayout;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.descriptor.repo.ReverseProxyDescriptor;
import org.artifactory.descriptor.repo.ReverseProxyRepoConfig;
import org.artifactory.model.FederatedRepoConfig;
import org.artifactory.model.LocalConfig;
import org.artifactory.model.ProxyRepoConfig;
import org.artifactory.model.RealRepoConfig;
import org.artifactory.model.ReleaseBundleRepoConfig;
import org.artifactory.model.RemoteRepoConfig;
import org.artifactory.model.RepoConfig;
import org.artifactory.model.VirtualRepoConfig;
import org.artifactory.model.XrayRepoConfig;
import org.artifactory.repo.CdnRedirectRepoConfig;
import org.artifactory.repo.DownloadRedirectRepoConfig;
import org.artifactory.repo.FederatedMemberConfiguration;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.repo.packagetype.DockerApiVersion;
import org.artifactory.repo.packagetype.ExternalDependenciesConfig;
import org.artifactory.repo.packagetype.configuration.BowerConfiguration;
import org.artifactory.repo.packagetype.configuration.CargoConfiguration;
import org.artifactory.repo.packagetype.configuration.CocoaPodsConfiguration;
import org.artifactory.repo.packagetype.configuration.ComposerConfiguration;
import org.artifactory.repo.packagetype.configuration.ConanConfiguration;
import org.artifactory.repo.packagetype.configuration.DebianConfiguration;
import org.artifactory.repo.packagetype.configuration.DockerConfiguration;
import org.artifactory.repo.packagetype.configuration.GitConfiguration;
import org.artifactory.repo.packagetype.configuration.GoConfiguration;
import org.artifactory.repo.packagetype.configuration.HelmConfiguration;
import org.artifactory.repo.packagetype.configuration.MavenConfiguration;
import org.artifactory.repo.packagetype.configuration.NpmConfiguration;
import org.artifactory.repo.packagetype.configuration.NuGetConfiguration;
import org.artifactory.repo.packagetype.configuration.P2Configuration;
import org.artifactory.repo.packagetype.configuration.PypiConfiguration;
import org.artifactory.repo.packagetype.configuration.TerraformConfiguration;
import org.artifactory.repo.packagetype.configuration.VcsConfiguration;
import org.artifactory.repo.packagetype.configuration.VcsGitConfiguration;
import org.artifactory.repo.packagetype.configuration.YumConfiguration;
import org.artifactory.repo.virtual.VirtualRepositoryConfigResolver;
import org.artifactory.ui.rest.model.admin.configuration.propertysets.PropertySetNameModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.AdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.BasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.CdnRedirectRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.DownloadRedirectRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.GeneralRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.ProxyAwareConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepoAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederationConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.MirrorMemberConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.releasebundles.ReleaseBundlesAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.releasebundles.ReleaseBundlesBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.releasebundles.ReleaseBundlesRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteCacheRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteNetworkRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.replication.local.LocalReplicationConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.replication.remote.RemoteReplicationConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.AlpineTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.BowerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.BuildInfoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CargoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ChefTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CocoaPodsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ComposerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ConanTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CondaTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CranTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DebTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DockerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GemsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GenericTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GitLfsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GradleTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.HelmTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.IvyTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.MavenTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.NpmTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.NugetTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.OpkgTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.P2TypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.PubTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.PuppetTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.PypiTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ReleaseBundlesRepoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.SbtTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.SwiftTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TerraformBackendTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TerraformTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.VagrantTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.VcsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.YumTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualSelectedRepository;
import org.artifactory.ui.rest.model.admin.configuration.repository.xray.XrayRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.reverseProxy.ReverseProxyRepoModel;
import org.artifactory.util.BowerPackageTypeConfigMapper;
import org.artifactory.util.CargoPackageTypeConfigMapper;
import org.artifactory.util.CocoaPodsPackageTypeConfigMapper;
import org.artifactory.util.CollectionUtils;
import org.artifactory.util.ComposerPackageTypeConfigMapper;
import org.artifactory.util.ConanPackageTypeConfigMapper;
import org.artifactory.util.DebianPackageTypeConfigMapper;
import org.artifactory.util.DockerPackageTypeConfigMapper;
import org.artifactory.util.ExternalDependenciesPackageTypeConfigMapper;
import org.artifactory.util.GoPackageTypeConfigMapper;
import org.artifactory.util.HelmPackageTypeConfigMapper;
import org.artifactory.util.MavenPackageTypeConfigMapper;
import org.artifactory.util.NpmPackageTypeConfigMapper;
import org.artifactory.util.NuGetPackageTypeConfigMapper;
import org.artifactory.util.P2PackageTypeConfigMapper;
import org.artifactory.util.PypiPackageTypeConfigMapper;
import org.artifactory.util.TerraformPackageTypeConfigMapper;
import org.artifactory.util.VcsPackageTypeConfigMapper;
import org.artifactory.util.YumPackageTypeConfigMapper;
import org.jfrog.common.config.validation.NotUnicode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.ConcurrentMap;
import java.util.stream.Collectors;

import static org.artifactory.descriptor.repo.RepoType.ReleaseBundles;
import static org.artifactory.util.FederatedMemberUtils.resolveUrl;
import static org.artifactory.util.PackageTypeConfigFields.FORCE_CONAN_AUTHENTICATION;
import static org.artifactory.util.PackageTypeConfigFields.FORCE_MAVEN_AUTHENTICATION;

/**
 * Utility class for converting descriptor to model
 *
 * @author Aviad Shikloshi
 */
@Slf4j
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class RepoConfigModelBuilder {
    private VirtualRepositoryConfigResolver resolver;
    public RepositoryConfigService repositoryConfigService;
    private CentralConfigService centralConfig;
    private AddonsManager addonsManager;

    @Autowired
    public void setResolver(VirtualRepositoryConfigResolver resolver) {
        this.resolver = resolver;
    }

    @Autowired
    public void setRepositoryConfigService(RepositoryConfigService repositoryConfigService) {
        this.repositoryConfigService = repositoryConfigService;
    }

    @Autowired
    public void setCentralConfig(CentralConfigService centralConfig) {
        this.centralConfig = centralConfig;
    }

    @Autowired
    public void setAddonsManager(AddonsManager addonsManager) {
        this.addonsManager = addonsManager;
    }

    /**
     * Populate model configuration from local repository config
     */
    public void populateLocalDescriptorValuesToModel(LocalConfig localConfig, LocalRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(localConfig);
        LocalBasicRepositoryConfigModel basic = createLocalBasicConfig(localConfig);
        LocalAdvancedRepositoryConfigModel advanced = createLocalAdvancedConfig(localConfig);
        TypeSpecificConfigModel typeSpecific =
                createLocalTypeSpecific(localConfig.getPackageType(), localConfig);

        model.setGeneral(general);
        model.setBasic(basic);
        model.setAdvanced(advanced);
        model.setTypeSpecific(typeSpecific);

        //the same behaviour as release bundle v1
        if (localConfig.getPackageType() != ReleaseBundles) {
            List<LocalReplicationDescriptor> replicationDescriptors = centralConfig.getDescriptor()
                    .getMultiLocalReplications(localConfig.getKey());
            List<LocalReplicationConfigModel> replications = replicationDescriptors.stream()
                    .map(this::createLocalReplicationConfig)
                    .collect(Collectors.toList());
            model.setReplications(replications);
        }
    }

    /**
     * Populate model configuration from federated repository config
     */
    public void populateFederatedDescriptorValuesToModel(FederatedRepoConfig federatedRepoConfig,
            FederatedRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(federatedRepoConfig);
        LocalBasicRepositoryConfigModel basic = createLocalBasicConfig(federatedRepoConfig);
        FederatedRepoAdvancedRepositoryConfigModel advanced = createFederatedRepoAdvancedConfig(federatedRepoConfig);
        TypeSpecificConfigModel typeSpecific =
                createLocalTypeSpecific(federatedRepoConfig.getPackageType(), federatedRepoConfig);

        List<MirrorMemberConfigModel> mirrorMemberConfigModels =
                federatedRepoConfig.getRepoTypeConfig().getFederatedMembers().stream()
                        .map(federatedMember -> new FederatedMemberConfiguration(resolveUrl(federatedMember),
                                federatedMember.getMode(), federatedMember.isEnabled()))
                        .map(this::toMemberConfigModel)
                        .toList();

        FederationConfigModel federationConfigModel = new FederationConfigModel();
        federationConfigModel.setMirrors(mirrorMemberConfigModels);

        model.setFederation(federationConfigModel);

        model.setGeneral(general);
        model.setBasic(basic);
        model.setAdvanced(advanced);
        model.setTypeSpecific(typeSpecific);
    }

    private MirrorMemberConfigModel toMemberConfigModel(FederatedMemberConfiguration federatedMemberConfiguration) {
        MirrorMemberConfigModel memberConfigModel = new MirrorMemberConfigModel();
        memberConfigModel.setUrl(federatedMemberConfiguration.getUrl());
        memberConfigModel.setEnabled(federatedMemberConfiguration.getEnabled());
        return memberConfigModel;
    }

    /**
     * Populate model configuration from remote repository config
     */
    public void populateRemoteRepositoryConfigValuesToModel(RemoteRepoConfig remoteRepoConfig,
            RemoteRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(remoteRepoConfig);
        model.setGeneral(general);

        RemoteBasicRepositoryConfigModel basic = createRemoteBasicConfig(remoteRepoConfig);
        model.setBasic(basic);

        RemoteReplicationDescriptor replicationDescriptor = centralConfig.getDescriptor().getRemoteReplication(
                remoteRepoConfig.getKey());
        if (replicationDescriptor != null) {
            RemoteReplicationConfigModel replication = createRemoteReplicationConfigModel(replicationDescriptor);
            model.setReplications(Lists.newArrayList(replication));
        }
        RemoteAdvancedRepositoryConfigModel advanced = createRemoteAdvancedConfig(remoteRepoConfig);
        model.setAdvanced(advanced);
        TypeSpecificConfigModel typeSpecific =
                createRemoteTypeSpecific(remoteRepoConfig.getPackageType(), remoteRepoConfig);
        model.setTypeSpecific(typeSpecific);
    }

    /**
     * Populate model configuration from virtual repository config
     */
    public void populateVirtualRepositoryConfigValuesToModel(VirtualRepoConfig virtualRepoConfig,
            VirtualRepositoryConfigModel model) {
        // General
        GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        general.setRepoKey(virtualRepoConfig.getKey());
        model.setGeneral(general);

        // Basic
        VirtualBasicRepositoryConfigModel basic = new VirtualBasicRepositoryConfigModel();
        Optional.ofNullable(virtualRepoConfig.getBaseConfig().getRepoLayoutRef())
                .ifPresent(basic::setLayout);
        Optional.ofNullable(virtualRepoConfig.getSecurityConfig().getPrimaryKeyPairRef())
                .ifPresent(primaryKeyRef -> {
                    basic.setKeypair(primaryKeyRef);
                    basic.setPrimaryKeyPair(primaryKeyRef);
                });
        Optional.ofNullable(virtualRepoConfig.getSecurityConfig().getSecondaryKeyPairRef())
                .ifPresent(basic::setSecondaryKeyPair);
        basic.setPublicDescription(virtualRepoConfig.getBaseConfig().getDescription());
        basic.setInternalDescription(virtualRepoConfig.getBaseConfig().getNotes());
        basic.setExcludesPattern(virtualRepoConfig.getBaseConfig().getExcludesPattern());
        basic.setIncludesPattern(virtualRepoConfig.getBaseConfig().getIncludesPattern());
        basic.setResolvedRepositories(
                resolver.getOrderedRepos(virtualRepoConfig.getKey()).stream()
                        .map(VirtualSelectedRepository::new).collect(Collectors.toList()));
        basic.setSelectedRepositories(
                repositoryConfigService.findRepositoryConfigsByKeysInSpecificOrder(virtualRepoConfig).stream()
                        .map(VirtualSelectedRepository::new).collect(Collectors.toList()));
        Optional.ofNullable(virtualRepoConfig.getRepoTypeConfig().getDefaultDeploymentRepoRef())
                .ifPresent(basic::setDefaultDeploymentRepo);
        model.setBasic(basic);

        // Advanced
        VirtualAdvancedRepositoryConfigModel advanced = new VirtualAdvancedRepositoryConfigModel();
        advanced.setRetrieveRemoteArtifacts(
                virtualRepoConfig.getRepoTypeConfig().isArtifactoryRequestsCanRetrieveRemoteArtifacts());
        addReverseProxyConfig(advanced, virtualRepoConfig.getKey());
        advanced.setHideUnauthorizedResources(virtualRepoConfig.getSecurityConfig().isHideUnauthorizedResources());
        model.setAdvanced(advanced);

        // Type specific
        TypeSpecificConfigModel typeSpecific =
                createVirtualTypeSpecific(virtualRepoConfig.getPackageType(), virtualRepoConfig);
        model.setTypeSpecific(typeSpecific);
    }

    /**
     * Populate model configuration from local repository config
     */
    public void populateReleaseBundlesDescriptorValuesToModel(ReleaseBundleRepoConfig releaseBundleRepoConfig,
            ReleaseBundlesRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(releaseBundleRepoConfig);
        ReleaseBundlesBasicRepositoryConfigModel basic = createReleaseBundlesBasicConfig(releaseBundleRepoConfig);
        ReleaseBundlesAdvancedRepositoryConfigModel advanced =
                createReleaseBundlesAdvancedConfig(releaseBundleRepoConfig);
        ReleaseBundlesRepoTypeSpecificConfigModel typeSpecific = new ReleaseBundlesRepoTypeSpecificConfigModel();
        model.setGeneral(general);
        model.setBasic(basic);
        model.setAdvanced(advanced);
        model.setTypeSpecific(typeSpecific);
    }

    private LocalReplicationConfigModel createLocalReplicationConfig(LocalReplicationDescriptor replicationDescriptor) {
        LocalReplicationConfigModel replication = new LocalReplicationConfigModel();
        replication.setUrl(replicationDescriptor.getUrl());
        replication.setCronExp(replicationDescriptor.getCronExp());
        replication.setEnableEventReplication(replicationDescriptor.isEnableEventReplication());
        replication.setEnabled(replicationDescriptor.isEnabled());
        replication.setDisableProxy(replicationDescriptor.isDisableProxy());
        if (!replicationDescriptor.isDisableProxy()) {
            ProxyDescriptor proxyDescriptor = replicationDescriptor.getProxy();
            if (proxyDescriptor != null) {
                replication.setProxy(proxyDescriptor.getKey());
            }
        }
        replication.setSocketTimeout(replicationDescriptor.getSocketTimeoutMillis());
        replication.setSyncDeletes(replicationDescriptor.isSyncDeletes());
        replication.setSyncProperties(replicationDescriptor.isSyncProperties());
        replication.setSyncStatistics(replicationDescriptor.isSyncStatistics());
        replication.setCheckBinaryExistenceInFilestore(replicationDescriptor.isCheckBinaryExistenceInFilestore());
        replication.setUsername(replicationDescriptor.getUsername());
        replication.setPassword(replicationDescriptor.getPassword());
        replication.setIncludePathPrefixPattern(replicationDescriptor.getIncludePathPrefixPattern());
        replication.setExcludePathPrefixPattern(replicationDescriptor.getExcludePathPrefixPattern());
        return replication;
    }

    private RemoteReplicationConfigModel createRemoteReplicationConfigModel(
            RemoteReplicationDescriptor replicationDescriptor) {
        RemoteReplicationConfigModel replication = new RemoteReplicationConfigModel();
        replication.setEnabled(replicationDescriptor.isEnabled());
        replication.setCronExp(replicationDescriptor.getCronExp());
        replication.setSyncDeletes(replicationDescriptor.isSyncDeletes());
        replication.setSyncProperties(replicationDescriptor.isSyncProperties());
        replication.setEnableEventReplication(replicationDescriptor.isEnableEventReplication());
        replication.setIncludePathPrefixPattern(replicationDescriptor.getIncludePathPrefixPattern());
        replication.setExcludePathPrefixPattern(replicationDescriptor.getExcludePathPrefixPattern());
        return replication;
    }

    private GeneralRepositoryConfigModel createGeneralConfig(RepoConfig localConfig) {
        GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        general.setRepoKey(localConfig.getKey());
        return general;
    }

    private LocalBasicRepositoryConfigModel createLocalBasicConfig(LocalConfig localConfig) {
        LocalBasicRepositoryConfigModel basic = new LocalBasicRepositoryConfigModel();
        addSharedBasicConfigModel(basic, localConfig);
        basic.setXrayConfig(getXrayConfig(localConfig.getSecurityConfig().getXrayConfig()));
        return basic;
    }

    private RemoteBasicRepositoryConfigModel createRemoteBasicConfig(RemoteRepoConfig remoteRepoConfig) {
        RemoteBasicRepositoryConfigModel basic = new RemoteBasicRepositoryConfigModel();
        addSharedBasicConfigModel(basic, remoteRepoConfig);
        basic.setUrl(remoteRepoConfig.getRepoTypeConfig().getUrl());
        basic.setXrayConfig(getXrayConfig(remoteRepoConfig.getSecurityConfig().getXrayConfig()));
        basic.setContentSynchronisation(remoteRepoConfig.getRepoTypeConfig().getContentSynchronisation());
        String remoteRepoLayoutRef = remoteRepoConfig.getRepoTypeConfig().getRemoteRepoLayoutRef();
        RepoLayout remoteRepoLayout = centralConfig.getDescriptor().getRepoLayout(remoteRepoLayoutRef);
        if (remoteRepoLayout != null) {
            basic.setRemoteLayoutMapping(remoteRepoLayout.getName());
        }
        basic.setOffline(remoteRepoConfig.getRepoTypeConfig().isOffline());
        basic.setCurated(remoteRepoConfig.getRepoTypeConfig().isCurated());
        return basic;
    }

    private ReleaseBundlesBasicRepositoryConfigModel createReleaseBundlesBasicConfig(
            ReleaseBundleRepoConfig descriptor) {
        ReleaseBundlesBasicRepositoryConfigModel basic = new ReleaseBundlesBasicRepositoryConfigModel();
        addSharedBasicConfigModel(basic, descriptor);
        return basic;
    }

    private void addSharedBasicConfigModel(BasicRepositoryConfigModel basic, RepoConfig repoConfig) {
        basic.setPublicDescription(repoConfig.getBaseConfig().getDescription());
        basic.setInternalDescription(repoConfig.getBaseConfig().getNotes());
        basic.setIncludesPattern(repoConfig.getBaseConfig().getIncludesPattern());
        basic.setExcludesPattern(repoConfig.getBaseConfig().getExcludesPattern());
        Optional.ofNullable(repoConfig.getBaseConfig().getRepoLayoutRef()).ifPresent(basic::setLayout);
        Optional.ofNullable(repoConfig.getSecurityConfig().getPrimaryKeyPairRef())
                .ifPresent(primaryKeyPairRef -> {
                    basic.setPrimaryKeyPair(primaryKeyPairRef);
                    basic.setKeypair(primaryKeyPairRef);
                });
        Optional.ofNullable(repoConfig.getSecurityConfig().getSecondaryKeyPairRef())
                .ifPresent(basic::setSecondaryKeyPair);
    }

    private LocalAdvancedRepositoryConfigModel createLocalAdvancedConfig(LocalConfig localConfig) {
        LocalAdvancedRepositoryConfigModel advanced = new LocalAdvancedRepositoryConfigModel();
        if (localConfig.getPackageType() == ReleaseBundles) {
            //The same behaviour as release bundle v1.
            addSharedAdvancedConfigModel(advanced, localConfig);
        } else {
            populateRepoAdvancedConfig(localConfig, advanced);
        }
        return advanced;
    }

    private void populateRepoAdvancedConfig(LocalConfig localConfig,
            LocalAdvancedRepositoryConfigModel advanced) {
        addSharedAdvancedConfigModel(advanced, localConfig);
        advanced.setDownloadRedirectConfig(
                getDownloadRedirectConfig(localConfig.getRepoTypeConfig().getDownloadRedirectConfig()));

        if (localConfig.getRepoTypeConfig().getCdnRedirectRepoConfig() != null) {
            final CdnRedirectRepoConfigModel cdnRedirectRepoConfigModel = new CdnRedirectRepoConfigModel();
            cdnRedirectRepoConfigModel.setEnabled(
                    localConfig.getRepoTypeConfig().getCdnRedirectRepoConfig().isEnabled());
            advanced.setCdnRedirectRepoConfig(cdnRedirectRepoConfigModel);
        }
        advanced.setPriorityResolution(localConfig.getRepoTypeConfig().isPriorityResolution());
    }

    private FederatedRepoAdvancedRepositoryConfigModel createFederatedRepoAdvancedConfig(
            FederatedRepoConfig federatedRepoConfig) {
        FederatedRepoAdvancedRepositoryConfigModel advanced = new FederatedRepoAdvancedRepositoryConfigModel();
        populateRepoAdvancedConfig(federatedRepoConfig, advanced);
        populateProxyValues(federatedRepoConfig, advanced);
        return advanced;
    }


    private RemoteAdvancedRepositoryConfigModel createRemoteAdvancedConfig(RemoteRepoConfig remoteRepoConfig) {
        RemoteAdvancedRepositoryConfigModel advanced = new RemoteAdvancedRepositoryConfigModel();
        addSharedAdvancedConfigModel(advanced, remoteRepoConfig);
        advanced.setDownloadRedirectConfig(
                getDownloadRedirectConfig(remoteRepoConfig.getRepoTypeConfig().getDownloadRedirectConfig()));
        advanced.setPriorityResolution(remoteRepoConfig.getRepoTypeConfig().isPriorityResolution());
        advanced.setSignedUrlTtl(remoteRepoConfig.getSecurityConfig().getSignedUrlTtl());
        RemoteNetworkRepositoryConfigModel networkModel = createNetworkConfig(remoteRepoConfig);
        advanced.setNetwork(networkModel);
        RemoteCacheRepositoryConfigModel cacheConfig = createCacheConfig(remoteRepoConfig);
        advanced.setCache(cacheConfig);
        advanced.setStoreArtifactsLocally(remoteRepoConfig.getRepoTypeConfig().isStoreArtifactsLocally());
        advanced.setSynchronizeArtifactProperties(remoteRepoConfig.getRepoTypeConfig().isSynchronizeProperties());
        advanced.setHardFail(remoteRepoConfig.getRepoTypeConfig().isHardFail());
        advanced.setQueryParams(remoteRepoConfig.getRepoTypeConfig().getQueryParams());
        advanced.setShareConfiguration(remoteRepoConfig.getRepoTypeConfig().isShareConfiguration());
        advanced.setBlockMismatchingMimeTypes(remoteRepoConfig.getRepoTypeConfig().isBlockMismatchingMimeTypes());
        advanced.setBypassHeadRequests(remoteRepoConfig.getRepoTypeConfig().isBypassHeadRequests());
        advanced.setDisableUrlNormalization(remoteRepoConfig.getRepoTypeConfig().isDisableUrlNormalization());
        advanced.setPropagateQueryParams(remoteRepoConfig.getRepoTypeConfig().isPropagateQueryParams());
        final CdnRedirectRepoConfig cdnRedirectRepoConfig =
                remoteRepoConfig.getRepoTypeConfig().getCdnRedirectRepoConfig();
        if (cdnRedirectRepoConfig != null) {
            CdnRedirectRepoConfigModel cdnRedirectRepoConfigModel = new CdnRedirectRepoConfigModel();
            cdnRedirectRepoConfigModel.setEnabled(cdnRedirectRepoConfig.isEnabled());
            advanced.setCdnRedirectRepoConfig(cdnRedirectRepoConfigModel);
        }
        if (StringUtils.isNotBlank(remoteRepoConfig.getRepoTypeConfig().getMismatchingMimeTypesOverrideList())) {
            advanced.setMismatchingMimeTypesOverrideList(
                    Lists.newArrayList(
                            remoteRepoConfig.getRepoTypeConfig().getMismatchingMimeTypesOverrideList().split(",")));
        }
        return advanced;
    }

    private ReleaseBundlesAdvancedRepositoryConfigModel createReleaseBundlesAdvancedConfig(
            ReleaseBundleRepoConfig releaseBundleRepoConfig) {
        ReleaseBundlesAdvancedRepositoryConfigModel advanced = new ReleaseBundlesAdvancedRepositoryConfigModel();
        addSharedAdvancedConfigModel(advanced, releaseBundleRepoConfig);
        return advanced;
    }

    private void addSharedAdvancedConfigModel(AdvancedRepositoryConfigModel advanced,
            RealRepoConfig realRepoConfig) {
        advanced.setAllowContentBrowsing(realRepoConfig.getRepoTypeConfig().isArchiveBrowsingEnabled());
        advanced.setBlackedOut(realRepoConfig.getRepoTypeConfig().isBlackedOut());
        advanced.setSignedUrlTtl(realRepoConfig.getSecurityConfig().getSignedUrlTtl());
        List<@NotUnicode String> propertySetRefs = realRepoConfig.getRepoTypeConfig().getPropertySetRefs();
        List<PropertySet> propertySetsList = centralConfig.getDescriptor().getPropertySets().stream()
                .filter(propertySet -> propertySetRefs.contains(propertySet.getName())).toList();
        List<PropertySetNameModel> propertySetNameModelList = collectPropertySets(propertySetsList);
        advanced.setPropertySets(propertySetNameModelList);
        addReverseProxyConfig(advanced, realRepoConfig.getKey());
    }

    RemoteCacheRepositoryConfigModel createCacheConfig(RemoteRepoConfig remoteRepoConfig) {
        RemoteCacheRepositoryConfigModel cacheConfig = new RemoteCacheRepositoryConfigModel();
        cacheConfig.setKeepUnusedArtifactsHours(
                remoteRepoConfig.getRepoTypeConfig().getUnusedArtifactsCleanupPeriodHours());
        cacheConfig.setRetrievalCachePeriodSecs(remoteRepoConfig.getRepoTypeConfig().getRetrievalCachePeriodSecs());
        cacheConfig.setAssumedOfflineLimitSecs(remoteRepoConfig.getRepoTypeConfig().getAssumedOfflinePeriodSecs());
        cacheConfig.setMissedRetrievalCachePeriodSecs(
                remoteRepoConfig.getRepoTypeConfig().getMissedRetrievalCachePeriodSecs());
        cacheConfig.setMetadataRetrievalTimeoutSecs(
                remoteRepoConfig.getRepoTypeConfig().getMetadataRetrievalTimeoutSecs());
        return cacheConfig;
    }

    private RemoteNetworkRepositoryConfigModel createNetworkConfig(RemoteRepoConfig remoteRepoConfig) {
        RemoteNetworkRepositoryConfigModel networkModel = new RemoteNetworkRepositoryConfigModel();
        populateProxyValues(remoteRepoConfig, networkModel);
        networkModel.setLocalAddress(remoteRepoConfig.getRepoTypeConfig().getLocalAddress());
        networkModel.setUsername(remoteRepoConfig.getRepoTypeConfig().getUsername());
        networkModel.setPassword(remoteRepoConfig.getRepoTypeConfig().getPassword());
        networkModel.setSocketTimeout(remoteRepoConfig.getRepoTypeConfig().getSocketTimeoutMillis());
        networkModel.setLenientHostAuth(remoteRepoConfig.getRepoTypeConfig().isAllowAnyHostAuth());
        networkModel.setCookieManagement(remoteRepoConfig.getRepoTypeConfig().isEnableCookieManagement());
        networkModel.setSyncProperties(remoteRepoConfig.getRepoTypeConfig().isSynchronizeProperties());
        String clientTleCertificate = remoteRepoConfig.getRepoTypeConfig().getClientTlsCertificate();
        if (StringUtils.isNotBlank(clientTleCertificate) &&
                clientTleCertificate.contains(ArtifactWebstartAddon.SSL_CERT_ALIAS_PREFIX)) {
            clientTleCertificate = clientTleCertificate.substring(ArtifactWebstartAddon.SSL_CERT_ALIAS_PREFIX.length());
        }
        networkModel.setSelectedInstalledCertificate(clientTleCertificate);
        ArtifactWebstartAddon webstartAddon = addonsManager.addonByType(ArtifactWebstartAddon.class);
        List<String> certAliases = webstartAddon.getSslCertNames();
        if (CollectionUtils.notNullOrEmpty(certAliases)) {
            networkModel.setInstalledCertificatesList(certAliases);
        }
        return networkModel;
    }

    private void populateProxyValues(ProxyRepoConfig proxyRepoConfig, ProxyAwareConfigModel model) {
        boolean isDisabledProxy = proxyRepoConfig.isProxyDisabled();
        model.setDisableProxy(isDisabledProxy);
        if (isDisabledProxy) {
            return;
        }
        String proxyRef = proxyRepoConfig.getProxyRef();
        if (StringUtils.isBlank(proxyRef)) {
            ProxyDescriptor defaultProxy = centralConfig.getDescriptor().getDefaultProxy();
            if (defaultProxy != null) {
                model.setProxy(defaultProxy.getKey());
            }
        } else if (centralConfig.getDescriptor().getProxy(proxyRef) != null) {
            model.setProxy(proxyRef);
        } else {
            log.debug("Repository '{}' contains a reference to proxy '{}' which does not exist on the config " +
                    "descriptor", proxyRepoConfig.getKey(), proxyRef);
        }
    }

    private TypeSpecificConfigModel createLocalTypeSpecific(RepoType type, LocalConfig localConfig) {
        MavenTypeSpecificConfigModel mavenModel = null;
        TypeSpecificConfigModel model = null;
        switch (type) {
            case ReleaseBundles -> model = new ReleaseBundlesRepoTypeSpecificConfigModel();
            case Bower -> model = new BowerTypeSpecificConfigModel();
            case Cargo -> {
                CargoTypeSpecificConfigModel cargo = new CargoTypeSpecificConfigModel();
                CargoConfiguration cargoConfiguration =
                        CargoPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
                cargo.setAnonymousAccess(cargoConfiguration.isCargoAnonymousAccess());
                cargo.setInternalIndex(cargoConfiguration.isCargoInternalIndex());
                model = cargo;
            }
            case Chef -> model = new ChefTypeSpecificConfigModel();
            case CocoaPods -> model = new CocoaPodsTypeSpecificConfigModel();
            case Composer -> {
                ComposerTypeSpecificConfigModel composer = new ComposerTypeSpecificConfigModel();
                ComposerConfiguration composerConfiguration =
                        ComposerPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
                composer.setEnableComposerV1Indexing(composerConfiguration.isEnableComposerV1Indexing());
                model = composer;
            }
            case Conan -> {
                ConanTypeSpecificConfigModel conanType = new ConanTypeSpecificConfigModel();
                populateSharedConanValues(conanType, localConfig);
                model = conanType;
            }
            case Docker -> {
                DockerTypeSpecificConfigModel dockerType = new DockerTypeSpecificConfigModel();
                DockerConfiguration dockerConfiguration =
                        DockerPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
                DockerApiVersion dockerApiVersion = dockerConfiguration.getDockerApiVersion();
                if (dockerApiVersion != null) {
                    dockerType.setDockerApiVersion(dockerApiVersion);
                }
                dockerType.setMaxUniqueTags(dockerConfiguration.getMaxUniqueTags());
                dockerType.setBlockPushingSchema1(dockerConfiguration.isBlockPushingSchema1());
                dockerType.setDockerTagRetention(dockerConfiguration.getDockerTagRetention());
                model = dockerType;
            }
            case Helm -> model = new HelmTypeSpecificConfigModel();
            case Alpine -> model = new AlpineTypeSpecificConfigModel();
            case Go -> model = new GoTypeSpecificConfigModel();
            case CRAN -> model = new CranTypeSpecificConfigModel();
            case Conda -> model = new CondaTypeSpecificConfigModel();
            case NuGet -> {
                NugetTypeSpecificConfigModel nugetType = new NugetTypeSpecificConfigModel();
                populateSharedNuGetValues(nugetType, localConfig);
                nugetType.setMaxUniqueSnapshots(localConfig.getRepoTypeConfig().getMaxUniqueSnapshots());
                model = nugetType;
            }
            case Npm -> model = new NpmTypeSpecificConfigModel();
            case Pypi -> model = new PypiTypeSpecificConfigModel();
            case Puppet -> model = new PuppetTypeSpecificConfigModel();
            case Vagrant -> model = new VagrantTypeSpecificConfigModel();
            case GitLfs -> model = new GitLfsTypeSpecificConfigModel();
            case Debian -> {
                DebTypeSpecificConfigModel debType = new DebTypeSpecificConfigModel();
                DebianConfiguration debianConfiguration =
                        DebianPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
                debType.setTrivialLayout(debianConfiguration.isDebianTrivialLayout());
                debType.setOptionalIndexCompressionFormats(debianConfiguration.getOptionalIndexCompressionFormats());
                debType.setDdebSupported(debianConfiguration.isDdebSupported());
                model = debType;
            }
            case Opkg -> model = new OpkgTypeSpecificConfigModel();
            case YUM -> {
                YumTypeSpecificConfigModel yumType = new YumTypeSpecificConfigModel();
                YumConfiguration yumConfiguration =
                        YumPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
                yumType.setMetadataFolderDepth(yumConfiguration.getYumRootDepth());
                yumType.setGroupFileNames(yumConfiguration.getYumGroupFileNames());
                yumType.setAutoCalculateYumMetadata(yumConfiguration.isCalculateYumMetadata());
                yumType.setEnableFileListsIndexing(yumConfiguration.isEnableFileListsIndexing());
                model = yumType;
            }
            case Gems -> model = new GemsTypeSpecificConfigModel();
            case Generic -> model = new GenericTypeSpecificConfigModel();
            case Maven -> mavenModel = new MavenTypeSpecificConfigModel();
            case Gradle -> mavenModel = new GradleTypeSpecificConfigModel();
            case Ivy -> mavenModel = new IvyTypeSpecificConfigModel();
            case Pub -> model = new PubTypeSpecificConfigModel();
            case Swift -> model = new SwiftTypeSpecificConfigModel();
            case SBT -> mavenModel = new SbtTypeSpecificConfigModel();
            case Terraform -> {
                TerraformTypeSpecificConfigModel typeSpecificConfigModel = new TerraformTypeSpecificConfigModel();
                TerraformConfiguration terraformConfiguration =
                        TerraformPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
                typeSpecificConfigModel.setTerraformType(terraformConfiguration.getTerraformType().getName());
                model = typeSpecificConfigModel;
            }
            case TerraformBackend -> model = new TerraformBackendTypeSpecificConfigModel();
            case BuildInfo -> model = new BuildInfoTypeSpecificConfigModel();
        }
        if (model != null) {
            return model;
        }
        // We will get here only if our model is maven / gradle / ivy / sbt and we populate the values
        populateMavenLocalValues(mavenModel, localConfig);
        return mavenModel;
    }

    TypeSpecificConfigModel createRemoteTypeSpecific(RepoType type, RemoteRepoConfig remoteRepoConfig) {
        MavenTypeSpecificConfigModel mavenModel = null;
        TypeSpecificConfigModel model = null;
        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        switch (type) {
            case Maven -> mavenModel = new MavenTypeSpecificConfigModel();
            case Gradle -> mavenModel = new GradleTypeSpecificConfigModel();
            case Ivy -> mavenModel = new IvyTypeSpecificConfigModel();
            case SBT -> mavenModel = new SbtTypeSpecificConfigModel();
            case P2 -> mavenModel = new P2TypeSpecificConfigModel();
            case Debian -> {
                DebTypeSpecificConfigModel debType = new DebTypeSpecificConfigModel();
                DebianConfiguration debianConfiguration =
                        DebianPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                debType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                debType.setDdebSupported(debianConfiguration.isDdebSupported());
                model = debType;
            }
            case Opkg -> {
                OpkgTypeSpecificConfigModel opkgType = new OpkgTypeSpecificConfigModel();
                opkgType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = opkgType;
            }
            case Docker -> {
                DockerTypeSpecificConfigModel dockerType = new DockerTypeSpecificConfigModel();
                dockerType.setEnableTokenAuthentication(
                        remoteRepoConfig.getRepoTypeConfig().isEnableTokenAuthentication());
                dockerType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                dockerType.setRetrievalCachePeriodSecs(
                        remoteRepoConfig.getRepoTypeConfig().getRetrievalCachePeriodSecs());
                DockerConfiguration dockerConfiguration =
                        DockerPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                ExternalDependenciesConfig dockerExternalDependencies = dockerConfiguration.getExternalDependencies();
                if (dockerExternalDependencies != null) {
                    dockerType.setEnableForeignLayersCaching(dockerExternalDependencies.isEnabled());
                    dockerType.setExternalPatterns(dockerExternalDependencies.getPatterns());
                }
                dockerType.setBlockPushingSchema1(dockerConfiguration.isBlockPushingSchema1());
                model = dockerType;
            }
            case NuGet -> {
                NugetTypeSpecificConfigModel nugetType = new NugetTypeSpecificConfigModel();
                populateSharedNuGetValues(nugetType, remoteRepoConfig);
                NuGetConfiguration nuGetConfiguration =
                        NuGetPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                if (nuGetConfiguration != null) {
                    nugetType.setDownloadContextPath(nuGetConfiguration.getDownloadContextPath());
                    nugetType.setFeedContextPath(nuGetConfiguration.getFeedContextPath());
                    nugetType.setV3FeedUrl(nuGetConfiguration.getV3FeedUrl());
                    nugetType.setSymbolServerUrl(nuGetConfiguration.getSymbolServerUrl());
                }
                nugetType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = nugetType;
            }
            case Npm -> {
                NpmTypeSpecificConfigModel npmType = new NpmTypeSpecificConfigModel();
                npmType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = npmType;
            }
            case Pypi -> {
                PypiTypeSpecificConfigModel pypiType = new PypiTypeSpecificConfigModel();
                pypiType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                PypiConfiguration pyPIConfiguration =
                        PypiPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                if (pyPIConfiguration != null) {
                    pypiType.setRegistryUrl(pyPIConfiguration.getPyPIRegistryUrl());
                    pypiType.setRepositorySuffix(pyPIConfiguration.getRepositorySuffix());
                }
                model = pypiType;
            }
            case Puppet -> {
                PuppetTypeSpecificConfigModel puppetType = new PuppetTypeSpecificConfigModel();
                puppetType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = puppetType;
            }
            case Chef -> {
                ChefTypeSpecificConfigModel chefType = new ChefTypeSpecificConfigModel();
                chefType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = chefType;
            }
            case VCS -> {
                VcsTypeSpecificConfigModel vcsType = new VcsTypeSpecificConfigModel();
                populateVcsValues(vcsType, remoteRepoConfig);
                model = vcsType;
            }
            case Bower -> {
                BowerTypeSpecificConfigModel bowerType = new BowerTypeSpecificConfigModel();
                populateVcsValues(bowerType, remoteRepoConfig);
                BowerConfiguration bowerConfiguration =
                        BowerPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                if (bowerConfiguration != null) {
                    bowerType.setRegistryUrl(bowerConfiguration.getBowerRegistryUrl());
                }
                model = bowerType;
            }
            case Go -> {
                GoTypeSpecificConfigModel goType = new GoTypeSpecificConfigModel();
                populateVcsValues(goType, remoteRepoConfig);
                goType.setMissedRetrievalCachePeriodSecs(
                        remoteRepoConfig.getRepoTypeConfig().getMissedRetrievalCachePeriodSecs());
                model = goType;
            }
            case CRAN -> model = new CranTypeSpecificConfigModel();
            case Conan -> {
                ConanTypeSpecificConfigModel conanType = new ConanTypeSpecificConfigModel();
                ConanConfiguration conanConfiguration =
                        ConanPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                conanType.setForceConanAuthentication(conanConfiguration.isForceConanAuthentication());
                model = conanType;
            }
            case Conda -> model = new CondaTypeSpecificConfigModel();
            case CocoaPods -> {
                CocoaPodsTypeSpecificConfigModel podsType = new CocoaPodsTypeSpecificConfigModel();
                populateVcsValues(podsType, remoteRepoConfig);
                CocoaPodsConfiguration podsConfiguration =
                        CocoaPodsPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                ExternalDependenciesConfig podsExternalDependencies = podsConfiguration.getExternalDependencies();
                if (podsExternalDependencies != null) {
                    podsType.setEnableExternalDependencies(podsExternalDependencies.isEnabled());
                    podsType.setExternalPatterns(podsExternalDependencies.getPatterns());
                }
                podsType.setSpecsRepoUrl(podsConfiguration.getCocoaPodsSpecsRepoUrl());
                podsType.setSpecsRepoProvider(podsConfiguration.getSpecRepoProvider());
                model = podsType;
            }
            case Gems -> {
                GemsTypeSpecificConfigModel gemsType = new GemsTypeSpecificConfigModel();
                gemsType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = gemsType;
            }
            case Generic -> {
                GenericTypeSpecificConfigModel genericType = new GenericTypeSpecificConfigModel();
                genericType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = genericType;
            }
            case YUM -> {
                YumTypeSpecificConfigModel yumType = new YumTypeSpecificConfigModel();
                yumType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                yumType.setAutoCalculateYumMetadata(null);
                yumType.setMetadataFolderDepth(null);
                yumType.setGroupFileNames(null);
                model = yumType;
            }
            case GitLfs -> {
                GitLfsTypeSpecificConfigModel gitLfsType = new GitLfsTypeSpecificConfigModel();
                gitLfsType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = gitLfsType;
            }
            case Helm -> {
                HelmTypeSpecificConfigModel helmType = new HelmTypeSpecificConfigModel();

                if (!packageTypeConfig.isEmpty()) {
                    final HelmConfiguration helmConfiguration =
                            HelmPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                    helmType.setChartsBaseUrl(helmConfiguration.getChartsBaseUrl());
                    ExternalDependenciesConfig helmExternalDependencies = helmConfiguration.getExternalDependencies();
                    if (helmExternalDependencies != null) {
                        helmType.setEnableExternalDependencies(helmExternalDependencies.isEnabled());
                        helmType.setExternalPatterns(helmExternalDependencies.getPatterns());
                    }
                }
                model = helmType;
            }
            case Alpine -> model = new AlpineTypeSpecificConfigModel();
            case Composer -> {
                ComposerTypeSpecificConfigModel composerType = new ComposerTypeSpecificConfigModel();
                populateVcsValues(composerType, remoteRepoConfig);
                ComposerConfiguration composerConfiguration =
                        ComposerPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                if (composerConfiguration != null) {
                    composerType.setRegistryUrl(composerConfiguration.getComposerRegistryUrl());
                }
                model = composerType;
            }
            case Cargo -> {
                CargoTypeSpecificConfigModel cargoType = new CargoTypeSpecificConfigModel();
                CargoConfiguration cargoConfiguration =
                        CargoPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                GitConfiguration gitConfiguration = cargoConfiguration.getGit();
                if (gitConfiguration != null) {
                    cargoType.setRegistryUrl(gitConfiguration.getRegistryUrl());
                }
                cargoType.setAnonymousAccess(cargoConfiguration.isCargoAnonymousAccess());
                cargoType.setInternalIndex(cargoConfiguration.isCargoInternalIndex());
                model = cargoType;
            }
            case Pub -> model = new PubTypeSpecificConfigModel();
            case Terraform -> {
                TerraformTypeSpecificConfigModel terraformTypeSpecific = new TerraformTypeSpecificConfigModel();
                populateVcsValues(terraformTypeSpecific, remoteRepoConfig);
                TerraformConfiguration terraformConfiguration =
                        TerraformPackageTypeConfigMapper.INSTANCE.map(packageTypeConfig);
                if (terraformConfiguration != null) {
                    terraformTypeSpecific.setRegistryUrl(terraformConfiguration.getTerraformRegistryUrl());
                    terraformTypeSpecific.setProvidersUrl(terraformConfiguration.getTerraformProvidersUrl());
                }
                model = terraformTypeSpecific;
            }
            case Swift -> {
                SwiftTypeSpecificConfigModel swiftType = new SwiftTypeSpecificConfigModel();
                ExternalDependenciesConfig externalDependencies =
                        ExternalDependenciesPackageTypeConfigMapper.INSTANCE.map(
                                packageTypeConfig);
                if (externalDependencies != null) {
                    swiftType.setEnableExternalDependencies(externalDependencies.isEnabled());
                    swiftType.setExternalPatterns(externalDependencies.getPatterns());
                }
                swiftType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
                model = swiftType;
            }
        }
        if (model != null) {
            return model;
        }
        populateMavenRemoteValues(mavenModel, remoteRepoConfig);
        return mavenModel;
    }

    private TypeSpecificConfigModel createVirtualTypeSpecific(RepoType type, VirtualRepoConfig virtualRepoConfig) {
        TypeSpecificConfigModel typeSpecific = null;
        MavenTypeSpecificConfigModel mavenModel = null;
        switch (type) {
            case Maven -> {
                mavenModel = new MavenTypeSpecificConfigModel();
                if (virtualRepoConfig.getPackageTypeConfig() != null) {
                    mavenModel.setForceMavenAuthentication(Boolean.valueOf(
                            virtualRepoConfig.getPackageTypeConfig().get(FORCE_MAVEN_AUTHENTICATION.getKey())));
                }
            }
            case Chef -> {
                ChefTypeSpecificConfigModel chef = new ChefTypeSpecificConfigModel();
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    chef.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = chef;
            }
            case Gradle -> mavenModel = new GradleTypeSpecificConfigModel();
            case Go -> {
                GoTypeSpecificConfigModel goModel = new GoTypeSpecificConfigModel();
                GoConfiguration goConfiguration =
                        GoPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
                ExternalDependenciesConfig goExternalDependencies = goConfiguration.getExternalDependencies();
                if (goExternalDependencies != null) {
                    goModel.setEnableExternalDependencies(goExternalDependencies.isEnabled());
                    goModel.setExternalPatterns(goExternalDependencies.getPatterns());
                }
                typeSpecific = goModel;
            }
            case CRAN -> {
                CranTypeSpecificConfigModel cranTypeSpecificConfigModel = new CranTypeSpecificConfigModel();
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    cranTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = cranTypeSpecificConfigModel;
            }
            case Conan -> {
                ConanTypeSpecificConfigModel conanTypeSpecificConfigModel = new ConanTypeSpecificConfigModel();
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    conanTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }


                if (virtualRepoConfig.getPackageTypeConfig() != null) {
                    conanTypeSpecificConfigModel.setForceConanAuthentication(Boolean.valueOf(
                            virtualRepoConfig.getPackageTypeConfig().get(FORCE_CONAN_AUTHENTICATION.getKey())));
                }
                typeSpecific = conanTypeSpecificConfigModel;
            }
            case Conda -> {
                CondaTypeSpecificConfigModel condaTypeSpecificConfigModel = new CondaTypeSpecificConfigModel();
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    condaTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = condaTypeSpecificConfigModel;
            }
            case Debian -> {
                DebTypeSpecificConfigModel debianTypeSpecificConfigModel = new DebTypeSpecificConfigModel();
                DebianConfiguration debianConfiguration =
                        DebianPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    debianTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());

                    debianTypeSpecificConfigModel.setVirtualArchitectures(
                            Arrays.asList(debianConfiguration.getDebianDefaultArchitectures().split(",")));
                    debianTypeSpecificConfigModel
                            .setOptionalIndexCompressionFormats(
                                    debianConfiguration.getDebianOptionalIndexCompressionFormats());
                }
                debianTypeSpecificConfigModel.setDdebSupported(debianConfiguration.isDdebSupported());
                typeSpecific = debianTypeSpecificConfigModel;
            }
            case Helm -> {
                HelmTypeSpecificConfigModel helm = new HelmTypeSpecificConfigModel();
                HelmConfiguration helmConfiguration =
                        HelmPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    helm.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }
                helm.setUseNamespaces(helmConfiguration.isUseNamespaces());
                typeSpecific = helm;
            }
            case Alpine -> {
                AlpineTypeSpecificConfigModel alpine = new AlpineTypeSpecificConfigModel();
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    alpine.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = alpine;
            }
            case Ivy -> mavenModel = new IvyTypeSpecificConfigModel();
            case SBT -> mavenModel = new SbtTypeSpecificConfigModel();
            case P2 -> {
                P2TypeSpecificConfigModel p2 = new P2TypeSpecificConfigModel();
                populateVirtualP2Values(p2, virtualRepoConfig);
                mavenModel = p2;
            }
            case Gems -> typeSpecific = new GemsTypeSpecificConfigModel();
            case GitLfs -> typeSpecific = new GitLfsTypeSpecificConfigModel();
            case Npm -> {
                NpmTypeSpecificConfigModel npmType = new NpmTypeSpecificConfigModel();
                NpmConfiguration npmConfiguration =
                        NpmPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
                ExternalDependenciesConfig npmExternalDependencies = npmConfiguration.getExternalDependencies();
                if (npmExternalDependencies != null) {
                    npmType.setEnableExternalDependencies(npmExternalDependencies.isEnabled());
                    if (npmExternalDependencies.getRemoteRepoRef() != null) {
                        npmType.setExternalRemoteRepo(npmExternalDependencies.getRemoteRepoRef());
                    }
                    npmType.setExternalPatterns(npmExternalDependencies.getPatterns());
                }
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    npmType.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = npmType;
            }
            case Bower -> {
                BowerTypeSpecificConfigModel bowerType = new BowerTypeSpecificConfigModel();
                BowerConfiguration bowerConfiguration =
                        BowerPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
                ExternalDependenciesConfig bowerExternalDependencies = bowerConfiguration.getExternalDependencies();
                if (bowerExternalDependencies != null) {
                    bowerType.setEnableExternalDependencies(bowerExternalDependencies.isEnabled());
                    if (bowerExternalDependencies.getRemoteRepoRef() != null) {
                        bowerType.setExternalRemoteRepo(bowerExternalDependencies.getRemoteRepoRef());
                    }
                    bowerType.setExternalPatterns(bowerExternalDependencies.getPatterns());
                }
                typeSpecific = bowerType;
            }
            case NuGet -> {
                typeSpecific = new NugetTypeSpecificConfigModel();
                populateSharedNuGetValues((NugetTypeSpecificConfigModel) typeSpecific, virtualRepoConfig);
            }
            case Pypi -> typeSpecific = new PypiTypeSpecificConfigModel();
            case Pub -> typeSpecific = new PubTypeSpecificConfigModel();
            case Swift -> typeSpecific = new SwiftTypeSpecificConfigModel();
            case Puppet -> typeSpecific = new PuppetTypeSpecificConfigModel();
            case Docker -> {
                DockerTypeSpecificConfigModel dockerType = new DockerTypeSpecificConfigModel();
                DockerConfiguration dockerConfiguration =
                        DockerPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
                dockerType.setResolveDockerTagsByTimestamp(dockerConfiguration.isResolveDockerTagsByTimestamp());
                dockerType.setCachingLocalForeignLayersEnabled(
                        dockerConfiguration.isCachingLocalForeignLayersEnabled());
                typeSpecific = dockerType;
            }
            case YUM -> {
                YumTypeSpecificConfigModel yum = new YumTypeSpecificConfigModel();
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    yum.setVirtualRetrievalCachePeriodSecs(virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = yum;
            }
            case Generic -> typeSpecific = new GenericTypeSpecificConfigModel();
            case Composer -> {
                ComposerTypeSpecificConfigModel composer = new ComposerTypeSpecificConfigModel();
                if (virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig() != null) {
                    composer.setVirtualRetrievalCachePeriodSecs(
                            virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig()
                                    .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = composer;
            }
            case Terraform -> typeSpecific = new TerraformTypeSpecificConfigModel();
            case Cargo -> {
                CargoTypeSpecificConfigModel cargo = new CargoTypeSpecificConfigModel();
                CargoConfiguration cargoConfiguration =
                        CargoPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
                cargo.setAnonymousAccess(cargoConfiguration.isCargoAnonymousAccess());
                cargo.setInternalIndex(true); //virtual cargo repo supported only with internal index
                typeSpecific = cargo;
            }
        }
        if (typeSpecific != null) {
            return typeSpecific;
        }
        populateMavenVirtualValues(mavenModel, virtualRepoConfig);
        return mavenModel;
    }

    private void populateVcsValues(VcsTypeSpecificConfigModel vcsType, RemoteRepoConfig remoteRepoConfig) {
        vcsType.setMaxUniqueSnapshots(remoteRepoConfig.getRepoTypeConfig().getMaxUniqueSnapshots());
        VcsConfiguration vcsConf = VcsPackageTypeConfigMapper.INSTANCE.map(remoteRepoConfig.getPackageTypeConfig());
        vcsType.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
        if (vcsConf != null) {
            vcsType.setVcsType(vcsConf.getType());
            VcsGitConfiguration git = vcsConf.getGit();
            if (git != null) {
                vcsType.setGitProvider(git.getProvider());
                vcsType.setDownloadUrl(git.getDownloadUrl());
            }
        }
    }

    private void populateSharedMavenValues(MavenTypeSpecificConfigModel model, RealRepoConfig realRepoConfig) {
        model.setMaxUniqueSnapshots(realRepoConfig.getRepoTypeConfig().getMaxUniqueSnapshots());
        model.setHandleReleases(realRepoConfig.getRepoTypeConfig().isHandleReleases());
        model.setHandleSnapshots(realRepoConfig.getRepoTypeConfig().isHandleSnapshots());
        MavenConfiguration mavenConfiguration =
                MavenPackageTypeConfigMapper.INSTANCE.map(realRepoConfig.getPackageTypeConfig());
        model.setSuppressPomConsistencyChecks(mavenConfiguration.isSuppressPomConsistencyChecks());
    }

    private void populateMavenLocalValues(MavenTypeSpecificConfigModel model, LocalConfig localConfig) {
        populateSharedMavenValues(model, localConfig);
        model.setSnapshotVersionBehavior(localConfig.getRepoTypeConfig().getSnapshotVersionBehavior());
        model.setLocalChecksumPolicy(localConfig.getRepoTypeConfig().getChecksumPolicyType());
    }

    private void populateMavenRemoteValues(MavenTypeSpecificConfigModel model, RemoteRepoConfig remoteRepoConfig) {
        populateSharedMavenValues(model, remoteRepoConfig);
        MavenConfiguration mavenConfiguration =
                MavenPackageTypeConfigMapper.INSTANCE.map(remoteRepoConfig.getPackageTypeConfig());
        model.setEagerlyFetchJars(mavenConfiguration.isFetchJarsEagerly());
        model.setEagerlyFetchSources(mavenConfiguration.isFetchSourcesEagerly());
        model.setRemoteChecksumPolicy(remoteRepoConfig.getRepoTypeConfig().getChecksumPolicyType());
        model.setListRemoteFolderItems(remoteRepoConfig.getRepoTypeConfig().isListRemoteFolderItems());
        model.setRejectInvalidJars(mavenConfiguration.isRejectInvalidJars());
    }

    private void populateMavenVirtualValues(MavenTypeSpecificConfigModel model, VirtualRepoConfig virtualRepoConfig) {
        MavenConfiguration mavenConfiguration =
                MavenPackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
        model.setPomCleanupPolicy(mavenConfiguration.getPomRepositoryReferencesCleanupPolicy());
        model.setKeyPair(virtualRepoConfig.getSecurityConfig().getKeyPair());
    }

    private void populateVirtualP2Values(P2TypeSpecificConfigModel model, VirtualRepoConfig virtualRepoConfig) {
        P2Configuration p2Configuration =
                P2PackageTypeConfigMapper.INSTANCE.map(virtualRepoConfig.getPackageTypeConfig());
        if (p2Configuration.getUrls() == null) {
            return;
        }
        Map<String, String> urlToRepoKeyMap =
                getUrlToRepoKeyMapping(virtualRepoConfig.getRepoTypeConfig().getRepositoryRefs());
        List<P2Repo> p2Repos = Lists.newArrayList();
        p2Configuration.getUrls().forEach(url -> {
            if (StringUtils.startsWith(url, "local://")) {
                Optional.ofNullable(resolveLocalP2RepoFromUrl(url)).ifPresent(p2Repos::add);
            } else {
                urlToRepoKeyMap.keySet().stream()
                        .map(RepoConfigDescriptorBuilder::getUrlWithoutSubpath)
                        .filter(p2Url -> RepoConfigDescriptorBuilder.getUrlWithoutSubpath(url).equals(p2Url))
                        .findAny()
                        .ifPresent(containingUrl ->
                                p2Repos.add(new P2Repo(null, null, urlToRepoKeyMap.get(containingUrl), url)));
            }
        });
        model.setP2Repos(p2Repos);
        model.setForceP2Authentication(p2Configuration.isForceP2Authentication());
    }

    P2Repo resolveLocalP2RepoFromUrl(String url) {
        String rpp = StringUtils.removeStart(url, "local://");
        RepoPath repoPath = RepoPathFactory.create(rpp);
        LocalConfig localConfig =
                repositoryConfigService.getLocalOrFederatedRepoConfigByKey(repoPath.getRepoKey());
        if (localConfig != null) {
            return new P2Repo(null, null, repoPath.getRepoKey(), url);
        }
        return null;
    }

    private List<PropertySetNameModel> collectPropertySets(List<PropertySet> propertySetsList) {
        return propertySetsList.stream()
                .map(PropertySetNameModel::new)
                .collect(Collectors.toList());
    }

    /**
     * Creates a mapping of url -> remote repo key to help build the P2 model (maps 'maven group' repos only)
     */
    private Map<String, String> getUrlToRepoKeyMapping(List<@NotUnicode String> repositoryRefs) {
        ConcurrentMap<String, String> mapping = Maps.newConcurrentMap();
        Set<RepoConfig> repoConfigs =
                repositoryConfigService.findRepositoryConfigsByKeys(Sets.newHashSet(repositoryRefs));
        repoConfigs.stream()
                .filter(RemoteRepoConfig.class::isInstance)
                .filter(repoDescriptor -> repoDescriptor.getPackageType().isMavenGroup())
                .forEach(remoteRepoConfig ->
                        mapping.put(((RemoteRepoConfig) remoteRepoConfig).getRepoTypeConfig().getUrl(),
                                remoteRepoConfig.getKey()));
        return mapping;
    }

    private void populateSharedNuGetValues(NugetTypeSpecificConfigModel nuget, RepoConfig localConfig) {
        NuGetConfiguration nuGetConfiguration =
                NuGetPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
        nuget.setForceNugetAuthentication(nuGetConfiguration.isForceNugetAuthentication());
    }

    private void populateSharedConanValues(ConanTypeSpecificConfigModel conan, RepoConfig localConfig) {
        ConanConfiguration conanConfiguration =
                ConanPackageTypeConfigMapper.INSTANCE.map(localConfig.getPackageTypeConfig());
        conan.setForceConanAuthentication(conanConfiguration.isForceConanAuthentication());
    }

    private XrayRepoConfigModel getXrayConfig(XrayRepoConfig xrayRepoConfig) {
        XrayRepoConfigModel model = null;
        if (xrayRepoConfig != null) {
            model = new XrayRepoConfigModel();
            model.setEnabled(xrayRepoConfig.isEnabled());
            model.setDataTtl(xrayRepoConfig.getDataTtl());
        }
        return model;
    }

    private DownloadRedirectRepoConfigModel getDownloadRedirectConfig(
            DownloadRedirectRepoConfig downloadRedirectRepoConfig) {
        DownloadRedirectRepoConfigModel model = null;
        if (downloadRedirectRepoConfig != null) {
            model = new DownloadRedirectRepoConfigModel();
            model.setEnabled(downloadRedirectRepoConfig.isEnabled());
        }
        return model;
    }

    private void addReverseProxyConfig(AdvancedRepositoryConfigModel advanced, String repoKey) {
        CentralConfigDescriptor configDescriptor = centralConfig.getDescriptor();
        ReverseProxyDescriptor reverseProxyDescriptor = configDescriptor.getCurrentReverseProxy();
        if (reverseProxyDescriptor != null) {
            ReverseProxyRepoConfig reverseProxyRepoConfig = reverseProxyDescriptor.getReverseProxyRepoConfig(repoKey);
            advanced.setReverseProxy(reverseProxyToModel(reverseProxyRepoConfig, reverseProxyDescriptor));
        }
    }

    private ReverseProxyRepoModel reverseProxyToModel(ReverseProxyRepoConfig reverseProxy,
            ReverseProxyDescriptor reverseProxyDescriptor) {
        if (reverseProxy != null) {
            ReverseProxyRepoModel model = new ReverseProxyRepoModel();
            model.setConfigurationKey(reverseProxyDescriptor.getKey());
            model.setServerName(reverseProxy.getServerName());
            model.setServerPort(reverseProxy.getPort());
            return model;
        }
        return null;
    }
}
