package org.artifactory.ui.rest.model.admin.services.indexer;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.descriptor.index.IndexerDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.model.FederatedRepoConfig;
import org.artifactory.model.RepoConfig;
import org.jetbrains.annotations.NotNull;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Set;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class IndexerTest {

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testPopulateRepos() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        Set<RepoConfig> federatedRepoConfigs = prepareRepositoryService(context);
        Indexer indexer = new Indexer();
        indexer.populateRepos(null);
        Assert.assertEquals(indexer.getExcludedRepos().size(), federatedRepoConfigs.size());
    }

    @Test
    public void testToDescriptor() {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        Set<RepoConfig> federatedRepoConfigs = prepareRepositoryService(context);

        Indexer indexer = new Indexer();
        indexer.setIncludedRepos(List.of("fed-maven", "fed-gradle"));
        IndexerDescriptor indexerDescriptor = indexer.toDescriptor();
        assert indexerDescriptor.getIncludedRepositories() != null;
        Assert.assertEquals(indexerDescriptor.getIncludedRepositories().size(), federatedRepoConfigs.size());

    }

    private Set<RepoConfig> prepareRepositoryService(ArtifactoryContext context) {
        RepositoryService repositoryService = mock(RepositoryService.class);
        Set<RepoConfig> federatedRepoConfigs = populateFederatedRepoConfigs();
        when(repositoryService.getRepoConfigsByTypes(any())).thenReturn(federatedRepoConfigs);
        when(context.beanForType(RepositoryService.class)).thenReturn(repositoryService);
        return federatedRepoConfigs;
    }

    @NotNull
    private Set<RepoConfig> populateFederatedRepoConfigs() {
        FederatedRepoConfig federatedRepoConfig1 = FederatedRepoConfig.create("fed-maven", RepoType.Maven);
        FederatedRepoConfig federatedRepoConfig2 = FederatedRepoConfig.create("fed-gradle", RepoType.Gradle);
        return Set.of(federatedRepoConfig1, federatedRepoConfig2);
    }
}