package org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.fetch;

import org.artifactory.api.rest.common.model.continues.ContinueResult;
import org.artifactory.api.rest.common.model.continues.FetchFunction;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.rest.common.model.RestModel;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.RepositoryType;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.RestTreeNode;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.TreeFilter;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.repo.RepositoryNode;
import org.testng.annotations.Test;

import java.util.Collections;
import java.util.List;

import static org.artifactory.util.bundles.BundleConfigurationUtil.DEFAULT_RELEASE_BUNDLES_V2_REPOSITORY;
import static org.testng.Assert.assertEquals;

/**
 * @author Omri Ziv
 */
public class RootFetchByRepositoryTypeTest extends RootFetchStrategyTest {

    @Test
    public void fetchFunctionsAmount() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.size(), 8);
    }

    @Test
    public void fetchItems() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        ContinueResult<RepositoryNode> result = rootFetchByPackageType.fetchItems();
        List<RepositoryNode> repositoryNodes = result.getData();
        assertEquals(repositoryNodes.get(0).getRepoKey(), "release-bundles");
        assertEquals(repositoryNodes.get(1).getRepoKey(), DEFAULT_RELEASE_BUNDLES_V2_REPOSITORY);
        assertEquals(repositoryNodes.get(2).getRepoKey(), "project-" + DEFAULT_RELEASE_BUNDLES_V2_REPOSITORY);
    }

    @Test
    public void getByRepositoryType() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        final int localRepoFunctionIndex = 3;
        assertEquals(fetchFunctions.get(localRepoFunctionIndex).getOriginalSize().longValue(), 25L); //local
        ContinueResult<? extends RestModel> continueResult1 = fetchFunctions.get(localRepoFunctionIndex)
                .getFunction().apply(0, 30);
        assertEquals(continueResult1.getData().size(), 24);
    }

    @Test
    public void getByRepositoryTypeAndProject() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setProjectKey(PROJECT_A);
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        final int localRepoFunctionIndex = 3;
        assertEquals(fetchFunctions.get(localRepoFunctionIndex).getOriginalSize().longValue(), 7L); //local
        ContinueResult<? extends RestModel> continueResult = fetchFunctions.get(localRepoFunctionIndex)
                .getFunction().apply(0, 10);
        assertEquals(continueResult.getData().size(), 7);
    }

    @Test
    public void getByRepositoryTypeLimited() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 25); // local
        assertEquals(fetchFunctions.get(4).getOriginalSize().longValue(), 1); // remote
        assertEquals(fetchFunctions.get(5).getOriginalSize().longValue(), 0); // cache
        assertEquals(fetchFunctions.get(6).getOriginalSize().longValue(), 1); //virtual
        assertEquals(fetchFunctions.get(7).getOriginalSize().longValue(), 1); //federated
        ContinueResult<? extends RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 10);
        assertEquals(continueResult1.getData().size(), 10);
    }

    @Test
    public void getByRepositoryTypeLimitedAndProject() {
        createMocks();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setProjectKey(PROJECT_A);
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 7); // local
        assertEquals(fetchFunctions.get(4).getOriginalSize().longValue(), 1); // remote
        assertEquals(fetchFunctions.get(5).getOriginalSize().longValue(), 0); // cache
        assertEquals(fetchFunctions.get(6).getOriginalSize().longValue(), 1); //virtual
        ContinueResult<? extends RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 5);
        assertEquals(continueResult1.getData().size(), 5);
    }

    @Test
    public void getByRepositoryTypeFilteredByType() {
        createMocksWithLocalAndVirtual();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setRepositoryTypes(Collections.singletonList(RepositoryType.VIRTUAL));
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.size(), 4);
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 8);
        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 30);
        assertEquals(continueResult1.getData().size(), 8);
    }

    @Test
    public void getByRepositoryTypeFilteredByTypeAndProject() {
        createMocksWithLocalAndVirtual();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setRepositoryTypes(Collections.singletonList(RepositoryType.VIRTUAL));
        treeFilter.setProjectKey(PROJECT_A);
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.size(), 4);
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 1);
        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 30);
        assertEquals(continueResult1.getData().size(), 1);
    }

    @Test
    public void getByRepositoryTypeFilteredByTypeAndPackageType() {
        createMocksWithLocalAndVirtual();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setRepositoryTypes(Collections.singletonList(RepositoryType.VIRTUAL));
        treeFilter.setPackageTypes(Collections.singletonList(RepoType.Maven));
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.size(), 4);
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 8); // virtual

        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 30);
        assertEquals(continueResult1.getData().size(), 3);
    }

    @Test
    public void getByRepositoryTypeFilteredByTypeAndPackageTypeFindNone() {
        createMocksWithLocalAndVirtual();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setPackageTypes(Collections.singletonList(RepoType.NuGet));
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 23); // local
        assertEquals(fetchFunctions.get(5).getOriginalSize().longValue(), 0); // remote
        assertEquals(fetchFunctions.get(6).getOriginalSize().longValue(), 8); // virtual

        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 30);
        assertEquals(continueResult1.getData().size(), 0);
        ContinueResult<RepositoryNode> continueResult2 = fetchFunctions.get(5).getFunction().apply(0, 30);
        assertEquals(continueResult2.getData().size(), 0);
        ContinueResult<RepositoryNode> continueResult3 = fetchFunctions.get(6).getFunction().apply(0, 30);
        assertEquals(continueResult3.getData().size(), 0);
    }

    @Test
    public void getByRepositoryTypeFilteredByKey() {
        createMocksWithLocalAndVirtual();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setByRepoKey("001");
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 23); // local
        assertEquals(fetchFunctions.get(4).getOriginalSize().longValue(), 0); // remote
        assertEquals(fetchFunctions.get(5).getOriginalSize().longValue(), 0); // remote
        assertEquals(fetchFunctions.get(6).getOriginalSize().longValue(), 8); // virtual

        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 30);
        assertEquals(continueResult1.getData().size(), 3);
        ContinueResult<RepositoryNode> continueResult3 = fetchFunctions.get(5).getFunction().apply(0, 30);
        assertEquals(continueResult3.getData().size(), 0);
    }

    @Test
    public void getByRepositoryTypeFilteredByKeyAndProject() {
        createMocksWithLocalAndVirtual();
        TreeFilter treeFilter = new TreeFilter();
        treeFilter.setByRepoKey("001");
        treeFilter.setProjectKey(PROJECT_A);
        RootFetchByRepositoryType rootFetchByPackageType =
                new RootFetchByRepositoryType(treeFilter, RestTreeNode.getRepoOrder());
        List<FetchFunction<RepositoryNode>> fetchFunctions = rootFetchByPackageType.getFetchFunctions();
        assertEquals(fetchFunctions.get(3).getOriginalSize().longValue(), 7); // local
        assertEquals(fetchFunctions.get(4).getOriginalSize().longValue(), 0); // remote
        assertEquals(fetchFunctions.get(5).getOriginalSize().longValue(), 0); // remote
        assertEquals(fetchFunctions.get(6).getOriginalSize().longValue(), 1); // virtual

        ContinueResult<RepositoryNode> continueResult1 = fetchFunctions.get(3).getFunction().apply(0, 30);
        assertEquals(continueResult1.getData().size(), 1);
        ContinueResult<RepositoryNode> continueResult3 = fetchFunctions.get(5).getFunction().apply(0, 30);
        assertEquals(continueResult3.getData().size(), 0);
    }
}
