package org.artifactory.ui.rest.service.admin.security.crowdsso;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.sso.crowd.CrowdAddon;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.security.SecurityDescriptor;
import org.artifactory.descriptor.security.sso.CrowdSettings;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.ui.rest.model.admin.security.crowdsso.CrowdIntegration;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class GetCrowdIntegrationServiceTest {

    @Mock
    private CrowdAddon crowdAddon;
    @Mock
    CentralConfigService centralConfigService;
    @Mock
    AddonsManager addonsManager;
    @Mock
    MutableCentralConfigDescriptor centralConfigDescriptor;
    @Mock
    SecurityDescriptor securityDescriptor;
    @Mock
    CrowdSettings crowdSettings;

    private GetCrowdIntegrationService crowdIntegrationService;

    @Test
    public void getCrowdFromKeyWhenConfigured() {
        String customKey = "customKey";
        when(crowdAddon.getCrowdCustomTokenKey()).thenReturn(customKey);
        RestResponse<CrowdIntegration> response = new ArtifactoryRestResponse<>();

        crowdIntegrationService.execute(mock(ArtifactoryRestRequest.class), response);
        CrowdIntegration crowdIntegration = response.getIModel();
        Assert.assertEquals(crowdIntegration.getCustomCookieTokenKey(), customKey);
        verify(addonsManager, times(1)).addonByType(CrowdAddon.class);
        verify(crowdAddon, times(1)).getCrowdCustomTokenKey();
        verify(centralConfigService, times(1)).getMutableDescriptor();
        verify(centralConfigDescriptor, times(1)).getSecurity();
        verify(securityDescriptor, times(1)).getCrowdSettings();
    }

    @Test
    public void testInit(){
        CrowdSettings crowdSettings = new CrowdSettings();
        crowdSettings.setAllowUserToAccessProfile(true);
        crowdSettings.setApplicationName("ApplicationName");
        crowdSettings.setDirectAuthentication(true);
        crowdSettings.setEnableIntegration(true);
        crowdSettings.setNoAutoUserCreation(true);
        crowdSettings.setPassword("Password");
        crowdSettings.setServerUrl("ServerUrl");
        crowdSettings.setSessionValidationInterval(5);
        crowdSettings.setUseDefaultProxy(true);
        CrowdIntegration crowdIntegration = new CrowdIntegration(crowdSettings);
        Assert.assertTrue(EqualsBuilder.reflectionEquals(crowdIntegration,crowdSettings));
    }

    @Test
    public void testInitWithNull(){
        CrowdIntegration crowdIntegration = new CrowdIntegration(null);
        CrowdSettings expected = new CrowdSettings();
        Assert.assertTrue(EqualsBuilder.reflectionEquals(crowdIntegration,expected));
    }


    @Test
    public void getCrowdFromKeyWhenNotConfigured() {
        RestResponse<CrowdIntegration> response = new ArtifactoryRestResponse<>();

        crowdIntegrationService.execute(mock(ArtifactoryRestRequest.class), response);
        CrowdIntegration crowdIntegration = response.getIModel();
        Assert.assertNull(crowdIntegration.getCustomCookieTokenKey());
        verify(addonsManager, times(1)).addonByType(CrowdAddon.class);
        verify(crowdAddon, times(1)).getCrowdCustomTokenKey();
        verify(centralConfigService, times(1)).getMutableDescriptor();
        verify(centralConfigDescriptor, times(1)).getSecurity();
        verify(securityDescriptor, times(1)).getCrowdSettings();
    }

    @BeforeMethod
    private void initCrowd() {
        MockitoAnnotations.initMocks(this);
        when(addonsManager.addonByType(CrowdAddon.class)).thenReturn(crowdAddon);
        when(securityDescriptor.getCrowdSettings()).thenReturn(crowdSettings);
        when(centralConfigDescriptor.getSecurity()).thenReturn(securityDescriptor);
        when(centralConfigService.getMutableDescriptor()).thenReturn(centralConfigDescriptor);

        crowdIntegrationService = new GetCrowdIntegrationService();
        crowdIntegrationService.setAddonsManager(addonsManager);
        crowdIntegrationService.setCentralConfigService(centralConfigService);
    }

    @AfterMethod
    private void noMoreCalls() {
        Mockito.verifyNoMoreInteractions(crowdAddon, centralConfigService, addonsManager, centralConfigDescriptor,
                securityDescriptor);
    }

}