package org.artifactory.ui.rest.service.admin.security.ldap.ldapsettings;

import com.google.common.collect.Lists;
import org.artifactory.api.common.BasicStatusHolder;
import org.artifactory.descriptor.security.ldap.LdapSetting;
import org.artifactory.descriptor.security.ldap.SearchPattern;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import java.util.List;

import static org.artifactory.ui.rest.service.admin.security.ldap.ldapsettings.LdapSettingsValidator.INVALID_PASSWORD_MSG;
import static org.artifactory.ui.utils.RequestUtils.PASSWORD_PLACEHOLDER;

/**
 * @author Dor Solomon
 */
class LdapSettingsValidatorTest {

    private static final List<Character> forbiddenChars = Lists
            .newArrayList('/', '\\', '|', '?', '*', '"', '<', '>');

    @ParameterizedTest
    @ValueSource(booleans = {true, false})
    void testValidate(boolean update) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        LdapSetting ldapSetting = new LdapSetting();
        ldapSetting.setKey("");
        LdapSettingsValidator.validate(ldapSetting, statusHolder, update);
        Assertions.assertTrue(statusHolder.hasErrors());
        Assertions.assertEquals("XML names cannot be empty", statusHolder.getStatusMsg());
        statusHolder.reset();

        for (Character character : forbiddenChars) {
            ldapSetting.setKey("test" + character);
            LdapSettingsValidator.validate(ldapSetting, statusHolder, update);
            Assertions.assertEquals("XML names cannot contain the character \"" + character + "\"",
                    statusHolder.getStatusMsg());
            statusHolder.reset();
        }

        ldapSetting.setKey("1-test");
        LdapSettingsValidator.validate(ldapSetting, statusHolder, update);
        Assertions.assertEquals("XML names cannot begin with the character \"1\"", statusHolder.getStatusMsg());
        statusHolder.reset();

        ldapSetting.setKey("valid-1-key-2");
        LdapSettingsValidator.validate(ldapSetting, statusHolder, update);
        Assertions.assertFalse(statusHolder.hasErrors());
        statusHolder.reset();

        if (update) {
            SearchPattern searchPattern = new SearchPattern();
            searchPattern.setManagerPassword(PASSWORD_PLACEHOLDER);
            ldapSetting.setSearch(searchPattern);
            LdapSettingsValidator.validate(ldapSetting, statusHolder, true);
            Assertions.assertEquals(INVALID_PASSWORD_MSG, statusHolder.getStatusMsg());
            statusHolder.reset();
        }
    }
}
